#
#       TRI REDUCTION SUB ROUTINE
#
#       average images with average-sigma-clipping for its binned value
#
#       1998/05/31      K.Kawabata ver.2.10  unification
#       2005/04/07      M.Isogai   ver.2.20  for iraf v2.12
#
#
procedure triaverage( inlist, wlist, x1, x2, avefile, sigfile, pixfile, rejtab, r_sigma )

string inlist { prompt = "Input image list" }
string wlist { prompt = "Weight list ('none' is used to avoid)" }
int x1 { prompt = "start pixel for rejection bin" }
int x2 { prompt = "end pixel for rejection bin" }
string avefile { prompt = "Output averaged image" }
string sigfile { prompt = "Output sigma image" }
string pixfile { prompt = "Output pixel number image" }
string rejtab { prompt = "Rejectioned image table" }
real r_sigma { prompt = "Sigma clipping factor" }

begin

  int n_list, n_ok
  string fn_stack[400]
  int flag_fn[400]
  real c[400], cc[400], worg[400]
  real c_ave, c_sig
  file infn, wfn, outave, outsig, outpix
  real weightsum, weight
  int nx1, nx2
  string fn_rej
  real rsig
  string buf, buf1
  int i, j
  real imsig = 1.0
  string tmp_fn, tmp_list, tmp_wfn
  real jd, am

#--- initial setting

  infn = inlist
  buf = ""
  if( access( infn ) == 0 )
  {
#    printf( "Cannot find list input-list(%s).\n", inlist )
#    printf( "Something is wrong, and the cause would be complex.\n" )
#    printf( "You MUST check images and files in this group.\n" )
#    printf( "But, you can proceed this reduction with imitation images.\n" )
#    printf( "Proceed? [y/n] >> " )
#    i = scan( buf )
#    while( buf != "n" && buf != "y" )
#      i = scan( buf )
#    if( buf == "n" )
      bye
  }

  wfn = wlist
  nx1 = x1
  nx2 = x2
  if( ( nx2 - nx1 ) < 0 )
  {
    printf( "NX2(%d) is below NX1(%d).\n", nx2, nx1 )
    bye
  }
  if( (nx1*nx2) != 0 )
    imsig = 10 / sqrt( x2 - x1 )
  outave = avefile
  outsig = sigfile
  outpix = pixfile
  fn_rej = rejtab
  rsig = r_sigma
  tmp_fn = mktemp( "tri.tmp.fn." )
  tmp_list = mktemp( "tri.tmp.list." )
  tmp_wfn = "tri.tmp.weight"

  if( access( tmp_fn ) == 1 )
    delete( tmp_fn, ver- )
  if( access( tmp_wfn ) == 1 )
    delete( tmp_wfn, ver- )
  if( access( fn_rej ) == 1 )
    delete( fn_rej, ver- )

  if( access( outave//".fits" ) == 1 )
    imdelete( outave, ver- )
  if( access( outsig//".fits" ) == 1 )
    imdelete( outsig, ver- )
  if( access( outpix//".pl.fits" ) == 1 )
    imdelete( outpix//".pl", ver- )
  if( access( outpix//".pl" ) == 1 )
    delete( outpix//".pl", ver- )
  if( access( outpix//".fits" ) == 1 )
    imdelete( outpix, ver- )

  for( i=1; i<=400; i+=1 )
    flag_fn[i] = 0


#--- scan list file

  n_list = 0
  list = infn
  buf = ""
  while( fscan( list, buf ) != EOF )
  {
    if( buf == "" )
      next
    n_list += 1
    fn_stack[n_list] = buf
    flag_fn[n_list] = 1
  }
  n_ok = n_list

  if( wfn != "none" ){
#    count( wfn )
#    type( wfn )
    list = wfn
    for( i=1; i<=n_list; i+=1 )
    {
      if( fscan( list, weight ) == EOF )
      {
        printf( "Weight file error (line %d)\n", i )
        return
      }
      worg[i] = weight
      c[i] = 0.0
    }
  }

  if( nx1 != 0 && nx2 != 0 )
  {

#--- imstat, and calculate average and sigma value, and rejection

    if( access( tmp_fn ) == 1 )
      delete( tmp_fn, ver- )

    for( i=1; i<=n_list; i+=1 )
    {
      imstatistics( fn_stack[i]//"["//nx1//":"//nx2//"]",\
                    format-, fields="mean,stddev", >> tmp_fn )
      list = tmp_fn
      if( fscan( list, buf, buf1 ) != 2 )
      {
        printf( "Imstatistics-file read error for %s\n", fn_stack[i] )
        bye
      }
      delete( tmp_fn, ver- )
      c[i] = real( buf )
      cc[i] = real( buf1 )
    }

    for( j=2; j>=0; j-=1 )
    {
      c_ave = 0.0
      c_sig = 0.0
      n_ok = 0

      for( i=1; i<=n_list; i+=1 )
      {
        if( flag_fn[i] == 1 )
	{
          n_ok += 1
          c_ave += c[i]
          c_sig += cc[i]
        }
      }
      c_ave = c_ave / n_ok
#      for( i=1; i<=n_list; i+=1 )
#      {
#        c_sig += ( ( c[i] - c_ave ) * ( c[i] - c_ave ) )
#      }
      c_sig = c_sig / n_ok
#      c_sig = sqrt( c_sig )

#printf( "j=%d ave %e sig %e\n", j, c_ave, c_sig )

#--- make new list

      for( i=1; i<=n_list; i+=1 )
      {
        if( flag_fn[i] == 1 )
	{
#printf( "i=%d diff=%e thre=%e\n", i, abs( c[i] - c_ave ), ( rsig*c_sig*imsig*10.0**j ) )
          if( ( abs( c[i] - c_ave ) > ( rsig*c_sig*imsig*10.0**j ) ) && nx1 != 0 && nx2 != 0 )
	  {
            flag_fn[i] = 0
          }
        }
      }
    }
  }

  if( access( tmp_list ) == 1 )
    delete( tmp_list, ver- )
  if( access( tmp_wfn ) == 1 )
    delete( tmp_wfn, ver- )

  n_ok = 0
  jd = 0
  am = 0
  for( i=1; i<=n_list; i+=1 )
  {
    if( n_list <= 2 || flag_fn[i] == 1 )
    {
      printf( "%s\n", fn_stack[i], >> tmp_list )
      n_ok += 1;
      imgets( fn_stack[i], "MJD" )
      jd += real( imgets.value )
      imgets( fn_stack[i], "AIRMASS" )
      am += real( imgets.value )
      printf( "%f\n", worg[i], >> tmp_wfn )
    }
    else
      printf( "%s  %5.2f\n", fn_stack[i], ((c[i]-c_ave)/c_sig), >> fn_rej )
  }
  if( n_ok > 0 )
  {
    jd = jd / n_ok
    am = am / n_ok
  }

  if( access( tmp_list ) == 0 )
  {
    printf( "Warning: all images rejected. ave=%e sigma=%e\n", c_ave, c_sig )
    if( access( tmp_wfn ) == 1 )
      delete( tmp_wfn, ver- )
    bye
  }

  if( wfn != "none" ){
    if( access( tmp_wfn ) == 0 )
    {
      printf( "Missing temporal weight file. Skip.\n" )
    }
    delete( wfn, ver- )
    rename( tmp_wfn, wfn )
  }

#--- remake weight list

  if( wfn != "none" ){
    if( access( wfn ) == 0 )
    {
      printf( "Weight list '%s' not found. Weighting avoided\n", wfn )
      wfn = "none"
    }
    weightsum = 0.0
    list = wfn
    while( fscan( list, weight ) != EOF )
      weightsum += weight
    list = wfn
    while( fscan( list, weight ) != EOF )
      printf( "%f\n", ( weight / weightsum ), >> tmp_wfn )
    delete( wfn, ver- )
    rename( tmp_wfn, wfn )
  }

#--- combine

#if( wfn != "none" ){
#count( tmp_list )
#count( wfn )
#}

  if( n_ok < 2 )
  {
    if( wfn != "none" )
    {
      oimcombine( "@"//tmp_list, outave, combine="average",\
                 reject="none", project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="@"//wfn, lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1,\
	         sigma="", plfile=outpix )
    }
    else
    {
      oimcombine( "@"//tmp_list, outave, combine="average",\
                 reject="none", project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="none", lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1,\
	         sigma="", plfile=outpix, >& "dev$null" )
    }
    imarith( outave, "-", outave, outsig, pixtype="real",\
             calctype="real", ver- )
  }
  else
  {
#    if( n_ok < 10 )
#      oimcombine( "@"//tmp_list, outave, combine="average",\
#                 reject="avsigclip", mclip+, project-, outtype="real",
#                 offsets="none", masktype="none", scale="none",\
#                 zero="none", weight="none", lthreshold=INDEF,\
#                 hthreshold=INDEF, nlow=1, nhigh=1,\
#                 lsigma=rsig, hsigma=rsig, sigma=outsig, plfile=outpix )
#    else
    if( wfn != "none" )
    {
      oimcombine( "@"//tmp_list, outave, combine="average",\
                 reject="sigclip", mclip+, project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="@"//wfn, lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1,\
                 lsigma=rsig, hsigma=rsig,\
		 sigma=outsig, plfile=outpix )
    }
    else
    {
      oimcombine( "@"//tmp_list, outave, combine="average",\
                 reject="sigclip", mclip+, project-, outtype="real",
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="none", lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1,\
                 lsigma=rsig, hsigma=rsig,\
		 sigma=outsig, plfile=outpix, >& "dev$null" )
    }
  }
  if( n_ok > 0 )
  {
    hedit( outave, "MJD", jd, add+, delete-, verify-, show-, update+ ) 
    hedit( outave, "AIRMASS", am, add+, delete-, verify-, show-, update+ )
    hedit( outave, "N_AVE", n_ok, add+, delete-, verify-, show-, update+ )
    imcopy( outave, tmp_fn, ver- )
    imreplace( tmp_fn, n_ok, lower=INDEF, upper=INDEF )
    imarith( tmp_fn, "-", outpix//".pl", outpix, pixtype="real",\
             calctype="real", ver- )
    if( access( outpix//".pl.fits" ) == 1 )
      imdelete( outpix//".pl", ver- )
    if( access( outpix//".pl" ) == 1 )
      delete( outpix//".pl", ver- )
    imdelete( tmp_fn, ver- )
  }
  delete( tmp_list, ver- )

end
