#
#       TRI REDUCTION SUB ROUTINE
#
#       Binning : tribin.cl
#
#       1998/03/07      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10
#       2000/02/29      K.Kawabata  Ver.3.00
#       2005/04/07      M.Isogai    Ver.3.10 for iraf v2.12
#

procedure tribin

begin

  file listoflist
  string trilist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  int c, i, j, n, n_list
  file linetab
  file fnphot

  string binmode # 1:bin 2:mean

  string binsel, binfn, binfn1
  real consterr
  string refbin
  file fn
  string fn_stack[500]
  int n_image
  string fnobe1, fnobo1, fnobe, fnobo
  string fnnexte, fnnexto
  file tmp_fn, tmp_list, tmp_list2
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7
  string prmfile
  file logfn
  file okfile = "tri.tmp.tricbin"

  file caldir, fn_def, etcdir
#  real crpix, crval, cdelt

  int flag_erase     # 0:keep old images, 1:erase old images
  int flag_inquiry   # 1:'flag_overwrite' applys to all image
  int flag_overwrite # 0:skip 1:overwrite
  int flag_ver       # 0:no display 1:display
  int n_version      # IRAF version (210 or 211)

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "tri.tmp.time" ) == 1 )
    delete( "tri.tmp.time", ver- )
  time >> tri.tmp.time
  list = "tri.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "tri.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "---------" )
  print( " Binning" )
  print( "---------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "---------", >> logfn )
  print( " Binning", >> logfn )
  print( "---------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  fn_def = caldir//envget( "_defhname" )
  fnphot = caldir//envget( "_photmet" )

  tmp_fn = mktemp( "tri.tmp.image." )
  tmp_list = mktemp( "tri.tmp.list." )
  tmp_list2 = mktemp( "tri.tmp.list." )


#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Replace Mode

  if( envget( "_fg_mode" ) == "1" )
  {
    flag_inquiry = 1
    flag_overwrite = 1
  }
  else
  {
    if( envget( "_fg_mode" ) == "2" )
    {
      flag_inquiry = 1
      flag_overwrite = 0
    }
    else
    {
      if( envget( "_fg_mode" ) == "3" )
      {
        flag_inquiry = 0
        flag_overwrite = 0
      }
      else
      {
        printf( "'triinit' process is incomplete.(_fg_mode) Retry.\n" )
        printf( "'triinit' process is incomplete.(_fg_mode) Retry.\n",\
                >> logfn )
        bye
      }
    }
  }
    
#----- Erase/Reserve old image

  if( envget( "_fg_del2" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del2" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_del2) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_del2) Retry.\n", >> logfn )
      bye
    }
  }

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" || buf2 == "V2.13" || buf2 == "V2.14")
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'trired'\n" )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n" )
    printf( "IRAF version check failed.\n", >> logfn )
    printf( "Please input like next line, and restart 'trired'\n", >> logfn )
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n", >> logfn )
    bye
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//".   Please do trigroup." )
    print( "Cannot find "//listoflist//".   Please do trigroup.", >> logfn )
    return
  }

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL",\
             >> logfn )
      break
    }
    if( buf2 != "OBJ" && buf2 != "COM" )
      next
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" && shorttyp[i] != "COM" )
    {
      prmfile = trilist[i]//".prm"

      if( flag_ver == 1 )
        printf( "--- OBJECT LIST : %s ", trilist[i] )
      printf( "--- OBJECT LIST : %s ", trilist[i], >> logfn )

      list = prmfile
      if( fscan( list, binsel, consterr, refbin, binmode, buf, buf, buf ) != 7 )
      {
        printf( "Read error in '%s'\n", prmfile )
        printf( "Read error in '%s'\n", prmfile, >> logfn )
        bye
      }
      binfn = trilist[i]//".bintab"
      binfn1 = trilist[i]//".binew"

      if( flag_ver == 1 )
      {
        if( binmode == "1" )
         printf( " sum mode ---\n" )
        if( binmode == "2" )
         printf( " average mode ---\n" )
      }
      if( binmode == "1" )
        printf( " sum mode ---\n", >> logfn )
      if( binmode == "2" )
        printf( " average mode ---\n", >> logfn )

      if( access( binfn1 ) == 1 )
        delete( binfn1, ver- )

   #--- make bintable for constant error
   
      if( binsel == "1" )
      {
        if( refbin != "---" )
        {
          if( access( refbin ) == 0 )
          {
	    printf( "Cannot find '%s'. Do triparam and tribin again.\n",\
	            refbin )
	  }
          if( access( binfn ) == 1 )
	    delete( binfn, ver- )
	  copy( refbin, binfn )
        }
        else
	{
          if( access( binfn ) == 1 )
#	  {
#	    buf = ""
#	    printf( "Old bin-tab '%s' found. Remake? [y/n] >> ", binfn )
#	    printf( "Old bin-tab '%s' found. Remake? [y/n] >> ", binfn,
#	    >> logfn )
#	    c = scan( buf )
#	    while( buf != "y" && buf != "n" )
#	      c = scan( buf )
#	    if( buf == "y" )
#	    {
#	      delete( binfn, ver- )
#              if( access( binfn1 ) == 1 )
#                delete( binfn1, ver- )
#	    }
#	    else
#	      goto bintabcheck
#	  } 
          printf( "Now making binning table for const-err of %4.2f%%...\n",\
	          consterr )
          printf( "Now making binning table for const-err of %4.2f%%...\n",\
	          consterr, >> logfn )
          n_image = 0
          if( access( tmp_list ) == 1 )
	    delete( tmp_list, ver- )
          if( access( tmp_list2 ) == 1 )
	    delete( tmp_list2, ver- )
	  
          sed( 's/fits//', trilist[i], > tmp_list )
          list = tmp_list
          while( fscan( list, buf ) != EOF )
          {
	    if( access( buf//"od.e.fits" ) == 1 )
	    {
	      printf( "%sod.e.fits\n", buf, >> tmp_list2 )
              n_image += 1
	    }
	    if( access( buf//"od.o.fits" ) == 1 )
	    {
	      printf( "%sod.o.fits\n", buf, >> tmp_list2 )
              n_image += 1
	    }
          }
          delete( tmp_list, ver- )
  	  if( n_image == 0 )
	  {
	    printf( "Not found any 1-d image for %s. Please check.\n",\
	            trilist[i] ) 
	    printf( "Not found any 1-d image for %s. Please check.\n",\
	    trilist[i], >> logfn )
	    bye
	  }
	  if( access( tmp_fn//".fits" ) == 1 )
	    imdelete( tmp_fn, ver- )
	  if ( access( tmp_fn ) == 1 )
	    delete( tmp_fn, ver- )
	  imsum( "@"//tmp_list2, tmp_fn, pixtype="real", calctype="real",\
	         option="sum", low_reject=0.0, high_reject=0.0, ver- )
	  delete( tmp_list2, ver- )

	  triconerrbin( tmp_fn//".fits", consterr, binfn )

          if( access( binfn1 ) == 1 )
            delete( binfn1, ver- )
          imdelete( tmp_fn, ver- )
	}       
      }
    bintabcheck:
      
      if( access( binfn ) == 0 )
      {
	printf( "Binning factor file '%s' not found.\n", binfn )
	printf( "Binning factor file '%s' not found.\n", binfn, >> logfn )
	bye
      }

#---- main routine

      sed( 's/fits//', trilist[i], > tmp_list )

      n_image = 0
      list = tmp_list

      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]

        fnobe1 = fn//"od.e"
        fnobo1 = fn//"od.o"
        fnobe = fn//"bn.e"
        fnobo = fn//"bn.o"

        if( j < n_image )
	{
          fnnexte = fn_stack[j+1]//"bn.e"
          fnnexto = fn_stack[j+1]//"bn.o"
	}
	else
	{
          fnnexte = "none"
          fnnexto = "none"
	}

        if( access( fnobe1//".fits" ) == 0 || access( fnobo1//".fits" ) == 0 )
        {
          printf( "%s--(not exist. skip.)  ", fn )
          printf( "%s--(not exist. skip.)  ", fn, >> logfn )
          next
        }	

        if( access( fnobe//".fits" ) == 1 || access( fnobo//".fits" ) == 1 )
        {
          if( flag_overwrite == 0 )
          {
            if( fnnexte != "none" && ( access( fnnexte//".fits" ) == 0 ||\
	        access( fnnexto//".fits" ) == 0 ) )
            {
	      printf( "\n%s: Maybe insufficient proccess last time.\n", fn )
	      printf( "\n%s: Maybe insufficient proccess last time.\n",\
                      fn, >> logfn )
              goto inquiry
            }

            if( flag_inquiry == 1 )
            {
              if( flag_ver == 1 )
                printf( "%s--(already exists. skip.)  ", fn )
              printf( "%s--(already exists. skip.)  ", fn, >> logfn )
              next
            }
            else
	    {
              printf( "\n%s already exist.\n", fn )
              printf( "\n%s already exist.\n", fn, >> logfn )
          inquiry:
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> " )
              printf( "Overwrite? (apply-this-image-only:[y/n], all:[Y/N]) >> ", >> logfn )
              c = scan( buf )
              while( buf != "Y" && buf != "y" && buf != "n" && buf != "N" )
                c = scan( buf )
              if( buf == "Y" )
              {
                flag_inquiry = 1
                flag_overwrite = 1
              }
              if( buf == "N" )
              {
                flag_inquiry = 1
                flag_overwrite = 0
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
              if( buf == "n" )
              {
                if( flag_ver == 1 )
                  printf( "%s--(already exists. skip.)  ", fn )
                printf( "%s--(already exists. skip.)  ", fn, >> logfn )
                next
              }
            }
          }
          if( flag_ver == 1 )
            printf( "%s(overwrite)  ", fn )
          printf( "%s(overwrite)  ", fn, >> logfn )
        }
        else
        {
          if( flag_ver == 1 )
            printf( "%s  ", fn )
          printf( "%s  ", fn, >> logfn )
        }

#---- binning

#	imreplace( fnobe1, 0.0, lower=INDEF, upper=0.0 )
#	imreplace( fnobo1, 0.0, lower=INDEF, upper=0.0 )

        if( access( fnobe1 ) == 1 )
	  delete( fnobe1, ver- )
        if( access( fnobo1 ) == 1 )
	  delete( fnobo1, ver- )
        if( access( fnobe ) == 1 )
	  delete( fnobe, ver- )
        if( access( fnobo ) == 1 )
	  delete( fnobo, ver- )

        if( access( fnobe//".fits" ) == 1 )
	  imdelete( fnobe, ver- )
        if( access( fnobo//".fits" ) == 1 )
	  imdelete( fnobo, ver- )

        copy( fnobe1//".fits", fnobe//".fits", ver- )
	copy( fnobo1//".fits", fnobo//".fits", ver- )

	if( access( binfn1 ) == 0 )
	{
	  tricbin( fnobe1//".fits", binfn, etcdir, fnphot, fnobe//".fits", binmode, binfn1 )
	  if( access( okfile ) == 0 )
	  {
	    delete( binfn1, ver- )
	    next
	  }
	  tricbin( fnobo1//".fits", binfn, etcdir, fnphot, fnobo//".fits", binmode, binfn1 )
	  if( access( okfile ) == 0 )
	  {
	    delete( binfn1, ver- )
	    next
	  }
	}
	else
	{
	  tricbin( fnobe1//".fits", binfn, etcdir, fnphot, fnobe//".fits", binmode )
	  if( access( okfile ) == 0 )
	  {
	    next
	  }
	  tricbin( fnobo1//".fits", binfn, etcdir, fnphot, fnobo//".fits", binmode )
	  if( access( okfile ) == 0 )
	  {
	    next
	  }
	}

	delete( okfile, ver- )


	imreplace( fnobe//".fits", 0.0, lower=INDEF, upper=0.0 )
        tricoord( fnobe )
	imreplace( fnobo//".fits", 0.0, lower=INDEF, upper=0.0 )
        tricoord( fnobo )

        hedit( fnobe//".fits", "HISTORY", "Binning performed",
          delete-, verify-, show-, update+ )
        hedit( fnobo//".fits", "HISTORY", "Binning performed",
          delete-, verify-, show-, update+ )

      }
      if( flag_ver == 1 )
        printf( "\n" )
      printf( "\n", >> logfn )
    }
  }

end
