#
#       TRISPEC REDUCTION SUB ROUTINE
#
#       Averaging dark images: tridarkave.cl
#
#       2006/12/17      K.Kawabata
#

procedure tridarkave

begin

  int i, j, k, n_list, n_darklist
  file fn
  string fn_stack[500]
  int n_image
  string listoflist, darklist, tmp_list
  string outlist[200], outlistsub, outfile[200], outfilesub, outlisttmp[200]
  string trilist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  string shortdiaph
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  string buf=""
  int flag_mode # avaraging mode  0: each object  1: only exp-time
  real cnt, zerolevel, sig
  file rejectimages = "TRI.darkave.rej"
  real geadu = 0.0
  int pregain = 1  #for TRISPEC
  int postgain = 1
  file caldir, fn_def
  file logfn
  real rej_thres = 1000.0 # (electrons)

  int nccd = 0       # 1:TC215 2:SITe
  int flag_erase # 0:keep old images, 1:erase old images
  int flag_ver  # 0:no display 1:display

#--- initial settings
  logfn = envget( "_logfile" )

  if( access( "tri.tmp.time" ) == 1 )
    delete( "tri.tmp.time", ver- )
  time >> tri.tmp.time
  list = "tri.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "tri.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "-----------------------" )
  print( " Averaging dark images" )
  print( "-----------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "-----------------------", >> logfn )
  print( " Averaging dark images", >> logfn )
  print( "-----------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  darklist = envget( "_darklist" )
  caldir = envget( "_caldir" )
  fn_def = caldir//envget( "_defhname" )
#  nccd = 1        # for old 'tridefine.h'

  tmp_list = mktemp( "tri.tmp.list." )

  if( access( listoflist ) == 0 )
  {
    printf( "Cannot find %s  Entering  trigroup...\n", listoflist )
    printf( "Cannot find %s  Entering  trigroup...\n", listoflist, >> logfn )
    trigroup
  }

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "GEADU" )
    {
      geadu = real( buf3 )
     }
    if( buf2 == "NCCD" )
     {
      nccd = int( buf3 )
    }
  }
#   printf("1:geadu=%g nccd=%d\n",geadu,nccd)

  if( geadu == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    printf( "Read error in '%s'\n", fn_def, >> logfn )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n", >> logfn )
      bye
    }
  }

#----- Erase/Reserve old image

  if( envget( "_fg_del1" ) == "1" )
  {
    flag_erase = 1
  }
  else
  {
    if( envget( "_fg_del1" ) == "2" )
    {
      flag_erase = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_del1) Retry.\n" )
      printf( "'triinit' process is incomplete.(_fg_del1) Retry.\n",\
       >> logfn )
      bye
    }
  }

 
#----- averaging mode
#
#  printf( "Select mode  [1]:each group [2]:each object (ignore diaph/filer)\n" )
#  printf( "             [3]:only exp-time  (default [1])  >> " )
#  k = scan( buf )
#  while( k != 0 && buf != "1" && buf != "2" && buf != "3" )
#    k = scan( buf )

  flag_mode = int( envget( "_fg_dark" ) ) - 1


#----- Delete old dark list and dark images

  if( access( darklist ) == 1 )
  {
    printf( "DARK-grouping files already exist.  Re-group? [y/n] >> " )
    printf( "DARK-grouping files already exist.  Re-group? [y/n] >> ",\
     >> logfn )
    k = scan( buf )
    while( buf != "y" && buf != "n" )
      k = scan( buf )
    printf( "%s\n", buf, >> logfn )
    if( buf == "y" )
    {
      if( flag_ver == 1 )
        printf( "Deleting old dark list(%s)...\n", darklist )
      printf( "Deleting old dark list(%s)...\n", darklist, >> logfn )
      delete( darklist )

      sections( "DARK.*.fits", option="fullname", > tmp_list )

      if( sections.nimages > 0 )
      {
        if( flag_ver == 1 )
          printf( "Deleting old dark images...\n" )
        printf( "Deleting old dark images...\n", >> logfn )
        imdelete( "@"//tmp_list )
      }
      delete( tmp_list, ver- ) 

      sections( "TRI.DARK.*", option="fullname", > tmp_list )

      if( sections.nimages > 0 )
      {
        if( flag_ver == 1 )
          printf( "Deleting old dark lists...\n" )
        printf( "Deleting old dark lists...\n", >> logfn )
        delete( "@"//tmp_list )
      }
      delete( tmp_list ) 
    }
    else
      goto averaging
  }

#----- Scanning the list file (made by trigroup)

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file "//listoflist//" is not produced by TRIGROUP.CL" )
      print( "The list-file "//listoflist//" is not produced by TRIGROUP.CL",\
       >> logfn )
      break
    }
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- Grouping dark images

  n_darklist = 0
  for( i=1; i<=n_list; i+=1 )
  {
    shortdiaph = diaph[i]
    if( shorttyp[i] == "DAR" )
    {
      if( flag_mode == 0 )
      {
        outlistsub = "TRI.DARK."//title[i]//"_"//diaph[i]//"_"//\
                     exptime[i]//"_"//filter[i]
        outfilesub = "DARK."//title[i]//"_"//diaph[i]//"_"//\
                     exptime[i]//"_"//filter[i]
      }
      else
      {
        if( flag_mode == 1 )
        {
          outlistsub = "TRI.DARK."//title[i]//"_"//shortdiaph//"_"//exptime[i]
          outfilesub = "DARK."//title[i]//"_"//shortdiaph//"_"//exptime[i]
        }
        else
        {
          outlistsub = "TRI.DARK."//exptime[i]
          outfilesub = "DARK."//exptime[i]
        }
      }
      if( n_darklist == 0 )
        goto nextstep1
      else
      {
        k = 0
        for( j=1; j<=n_darklist; j+=1 )
        {
          if( outlistsub == outlist[j] )
          k = 1
        }
        if( k == 0 )
          goto nextstep1
      }
      goto nextstep2

nextstep1:
      n_darklist += 1
      outlist[n_darklist] = outlistsub
      outlisttmp[n_darklist] = outlistsub//".tmp"
      outfile[n_darklist] = outfilesub
      if( flag_mode == 0 )
        printf( "%12s %6s %7s %13s %s %s\n", outfilesub, title[i],\
	        exptime[i], diaph[i], filter[i], slit[i],  >> darklist )
      else
        if( flag_mode == 1 )
          printf( "%12s %6s %7s %7s %s %s\n", outfilesub, title[i],\
                  exptime[i], shortdiaph, filter[i], slit[i],  >> darklist )
	else
          printf( "%12s ------ %7s %13s %s %s\n", outfilesub,\
	          exptime[i], diaph[i], filter[i], slit[i],  >> darklist )

nextstep2:
      type( trilist[i], >> outlistsub )
    }
  }

averaging:

#----- Bias correction

  for( i=1; i<=n_darklist; i+=1 )
  {
    n_image = 0
    if( flag_ver == 1 )
      printf( "--- %15s --> %s ---\n", outlist[i], outfile[i] )
    printf( "--- %15s --> %s ---\n", outlist[i], outfile[i], >> logfn )

    if( access( outfile[i]//".fits" ) == 1 )
    {
      printf( "'%s' still exists. Skip.\n", outfile[i] )
      printf( "'%s' still exists. Skip.\n", outfile[i], >> logfn)
      next
    }
    list = outlist[i]
    while( fscan( list, buf ) != EOF ){
      n_image += 1
      fn_stack[n_image] = buf
    }

    for( j=1; j<=n_image; j+=1 )
    {
      fn = fn_stack[j]

      if( access( fn ) == 0  && access( fn//".fits" ) == 0 )
        next

#      printf("%d\n",nccd)


      imstatistics( fn, format-, fields="midpt,stddev", >> tmp_list )
      list = tmp_list
      if( fscan( list, cnt, sig ) != 2 )
      {
        printf( "Imstatistics-file read error for %s\n", fn )
        printf( "Imstatistics-file read error for %s\n", fn, >> logfn )
        next
      }
      delete( tmp_list, ver- )

#     if( nccd == 2 && sig > (pregain*postgain*geadu*rej_thres) )
      if( ( nccd == 1 || nccd == 2 ) && sig > (pregain*postgain*geadu*rej_thres) )
      {
        printf( "%s is rejected because of too large sigma of %f\n",\
	 fn, sig )
        printf( "%s is rejected because of too large sigma of %f\n",\
	 fn, sig, >> logfn )
        printf( "%s %f\n", fn, sig, >> rejectimages )
        next
      }

      if( j == 1 )
      {
        zerolevel = cnt
      }
      if( flag_ver == 1 )
        printf( "%15s  %7.2f %7.2f %6.2f subtracting %6.2f\n", fn,\
	 cnt, zerolevel, sig, (cnt-zerolevel) )
      printf( "%15s  %7.2f %7.2f %6.2f subtracting %6.2f\n", fn,\
       cnt, zerolevel, sig, (cnt-zerolevel), >> logfn )
      if( j > 1 && abs( cnt - zerolevel ) > 0.01 )
      {
        imarith( fn, "-", (cnt - zerolevel), fn, pixtype="real",\
	         calctype="real", ver- )
      }
      buf = outlisttmp[i]
      printf( "%s\n", fn, >> buf )
    }

    n_image = 0
    list = outlisttmp[i]
    while( fscan( list, buf ) != EOF ){
      n_image += 1
    }
    if( n_image == 0 )
    {
      print( outlisttmp[i], "is empty. Skip." )
      print( outlisttmp[i], "is empty. Skip.", >> logfn )
      next
    }

#----- Averaging(creating) dark images


    if( n_image < 3 )
      oimcombine( input="@"//outlisttmp[i], output=outfile[i],\
                 combine="average",\
                 reject="none", project-, outtype="real",\
                 offsets="none", masktype="none", scale="none",\
                 zero="none", weight="none", lthreshold=INDEF,\
                 hthreshold=INDEF, nlow=1, nhigh=1, >> logfn )
    else
        oimcombine( input="@"//outlisttmp[i], output=outfile[i],\
                   combine="average",\
                   reject="sigclip", project-, outtype="real",\
                   offsets="none", masktype="none", scale="none",\
                   zero="none", weight="none", lthreshold=INDEF,\
                   hthreshold=INDEF, nlow=1, nhigh=1, nkeep=1,\
                   mclip+, lsigma=2.0, hsigma=2.0, sigscale=0.1,\
		   >> logfn )

    if( flag_erase == 1 && access( outfile[i]//".fits" ) == 1 )
      imdelete( "@"//outlist[i], ver- )
    delete( outlisttmp[i] )

  }

end
