#
#       TRI REDUCTION SUB ROUTINE
#
#       Grouping images refering its HWP angles : triiqugroup.cl
#
#       1997/05/02      K.Kawabata  Ver.0.10  for CRYOCAM manual reduc
#       1997/09/08      K.Kawabata  Ver.1.00  use 'TRI.list'
#       1998/04/20      K.Kawabata  Ver.2.00  unification
#       1998/05/31      K.Kawabata  Ver.2.10  bin/unbin
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#       2000/03/25      K.Kawabata  Ver.3.01  weighted mean
#

procedure triiqugroup

begin

  file fn, fnobe, fnobo
  string fn_stack[500]
  int n_image
  string fullkappa, kappahead

  file listoflist
  string trilist[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500]
  string slit[500]
  int c, i, j, k, n_list

  string iqulist, qlist, ulist, ilist
  string klist[4]
  string khead
  int n_q, n_u, n_i
  int n_k[4]
  string qumode
  real theta
  int theta_n, n_theta_n
  int hwppos
  int flag_last[4]
  string last_image[4]

  file logfn
  file tmp_list, tmp_fn, tmp_fn1
  string buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7

  string _klist = ".kap"

#--- initial settings

  logfn = envget( "_logfile" )

  if( access( "tri.tmp.time" ) == 1 )
    delete( "tri.tmp.time", ver- )
  time >> tri.tmp.time
  list = "tri.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "tri.tmp.time", ver- )

  print( "" )
  printf( "%s %s\n", buf1, buf2 )
  print( "-----------------------" )
  print( " Grouping by HWP Angle"
  print( "-----------------------" )
  print( "" )

  print( "", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  print( "-----------------------", >> logfn )
  print( " Grouping by HWP Angle", >> logfn )
  print( "-----------------------", >> logfn )
  print( "", >> logfn )

  listoflist = envget( "_rootlist" )
  iqulist = envget( "_iqulist" )

  tmp_list = mktemp( "tri.tmp.list." )

  if( access( iqulist ) == 1 )
    delete( iqulist, ver- )

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do trigroup" )
    print( "Cannot find "//listoflist//"   Please do trigroup", >> logfn )
    bye
  }

  n_list = 0
  list = listoflist

  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL",\
             >> logfn )
      break
    }
    if( buf2 != "OBJ" && buf2 != "COM" )
      next
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

#----- main routine ----

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] != "DAR" && shorttyp[i] != "BIA" &&\
        shorttyp[i] != "NEA" && shorttyp[i] != "COM")
    {
      kappahead = ""

      if( substr( diaph[i], 1, 3 ) == "2HL" )
      {
        if( substr( diaph[i], 4, 4 ) == "L" )
          kappahead = envget( "_up2hll" )
        if( substr( diaph[i], 4, 4 ) == "U" )
          kappahead = envget( "_up2hlu" )
      }
      if( substr( diaph[i], 1, 3 ) == "2S2" )
      {
        if( substr( diaph[i], 4, 4 ) == "L" )
          kappahead = envget( "_up2s2l" )
        if( substr( diaph[i], 4, 4 ) == "U" )
          kappahead = envget( "_up2s2u" )
      }
      if( kappahead == "" ){
        printf( "Unknown Diaph type in %s (%s)\n", trilist[i], diaph[i] )
        printf( "Unknown Diaph type in %s (%s)\n", trilist[i], diaph[i],\
                >> logfn )
	next
      }

      n_theta_n = 0
      n_q = 0
      n_u = 0
      n_i = 0

      khead = trilist[i]//_klist//"."
      for( k=1; k<=4; k+=1 )
      {
        n_k[k] = 0
        flag_last[k] = 0
	last_image[k] = ""
        klist[k] = khead//(k-1)
        if( access( klist[k] ) == 1 )
          delete( klist[k], ver- )
      }
      qlist = trilist[i]//envget( "_qlist" )
      ulist = trilist[i]//envget( "_ulist" )
      ilist = trilist[i]//envget( "_ilist" )

      if( access( qlist ) == 1 )
        delete( qlist, ver- )
      if( access( ulist ) == 1 )
        delete( ulist, ver- )
      if( access( ilist ) == 1 )
        delete( ilist, ver- )

      printf( "--- OBJECT LIST : %s ---\n", trilist[i] )
      printf( "--- OBJECT LIST : %s ---\n", trilist[i], >> logfn )
      sed( 's/fits//', trilist[i], > tmp_list )

      list = tmp_list
      n_image = 0
      while( fscan( list, buf ) != EOF )
      {
        n_image += 1
        fn_stack[n_image] = buf
      }
      delete( tmp_list, ver- )

      qumode = "SELF"

      for( j=1; j<=n_image; j+=1 )
      {
        fn = fn_stack[j]
	fnobe = fn//"bn.e"
	fnobo = fn//"bn.o"

	if( access( fnobe//".fits" ) == 0 )
        {
	  printf( "Not found '%s'. Skip.\n", fnobe )
	  printf( "Not found '%s'. Skip.\n", fnobe, >> logfn )
	  next
	}
	if( access( fnobo//".fits" ) == 0 )
        {
	  printf( "Not found '%s'. Skip.\n", fnobo )
	  printf( "Not found '%s'. Skip.\n", fnobo, >> logfn )
	  next
	}

        imgets( fnobe, 'POS-HWP' )
        theta = real( imgets.value )
        theta_n = int( theta / 22.499999 )

        hwppos = theta_n % 4
	fullkappa = kappahead//hwppos

        if( hwppos == 0 )
        {
	  if( flag_last[1] == 0 )
	  {
	    flag_last[1] = 1
	    n_theta_n += 1
	  }
	  last_image[1] = fn
	  if( flag_last[3] > 0 )
	  {
	    n_q += 1
	    buf = "00"//str( n_q )
	    tmp_fn = trilist[i]//".q."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fn, last_image[3],\
	            tmp_fn, >> qlist )
	  }

	  n_i += 1
          buf = "00"//str( n_i )
          tmp_fn = trilist[i]//".i1."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
          tmp_fn1 = trilist[i]//".i2."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
          printf( "%s  %s %s %s\n", fn, fullkappa, tmp_fn, tmp_fn1, >> ilist )
          printf( "%s  %s %s %s\n", fn, fullkappa, tmp_fn, tmp_fn1 )
          printf( "%s  %s %s %s\n", fn, fullkappa, tmp_fn, tmp_fn1, >> logfn )

	  n_k[1] = n_k[1] + 1
          buf = "00"//str( n_k[1] )
          tmp_fn = trilist[i]//".kap.0."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	  tmp_fn1 = klist[1]
          printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
        }

        if( hwppos == 1 )
        {
	  if( flag_last[2] == 0 )
	  {
	    flag_last[2] = 1
	    n_theta_n += 1
	  }
	  last_image[2] = fn
	  if( flag_last[4] > 0 )
	  {
	    n_u += 1
	    buf = "00"//str( n_u )
	    tmp_fn = trilist[i]//".u."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fn, last_image[4],\
	            tmp_fn, >> ulist )
	  }

	  n_k[2] = n_k[2] + 1
          buf = "00"//str( n_k[2] )
          tmp_fn = trilist[i]//".kap.1."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	  tmp_fn1 = klist[2]
          printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
        }

        if( hwppos == 2 )
        {
	  if( flag_last[3] == 0 )
	  {
	    flag_last[3] = 1
	    n_theta_n += 1
	  }
	  last_image[3] = fn
	  if( flag_last[1] > 0 )
	  {
	    n_q += 1
	    buf = "00"//str( n_q )
	    tmp_fn = trilist[i]//".q."//substr( buf, ( strlen( buf ) - 2 ),\
                     strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, last_image[1], fn,\
	            tmp_fn, >> qlist )
	  }

	  n_k[3] = n_k[3] + 1
          buf = "00"//str( n_k[3] )
          tmp_fn = trilist[i]//".kap.2."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	  tmp_fn1 = klist[3]
          printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
        }

        if( hwppos == 3 )
        {
	  if( flag_last[4] == 0 )
	  {
	    flag_last[4] = 1
	    n_theta_n += 1
	  }
	  last_image[4] = fn
	  if( flag_last[2] > 0 )
	  {
	    n_u += 1
	    buf = "00"//str( n_u )
	    tmp_fn = trilist[i]//".u."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, last_image[2], fn,\
	            tmp_fn, >> ulist )
	  }

	  n_k[4] = n_k[4] + 1
          buf = "00"//str( n_k[4] )
          tmp_fn = trilist[i]//".kap.3."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	  tmp_fn1 = klist[4]
          printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
        }
      }

#----- if no pair exists.

      if( n_q == 0 )
      {
        qumode = "KAPPA"
        printf( "Not found any pair for q. Then execute q for each image.\n" )
        printf( "Not found any pair for q. Then execute q for each image.\n",\
                >> logfn )

        n_k[1] = 0
        klist[1] = khead//"0"
        if( access( klist[1] ) == 1 )
          delete( klist[1], ver- )
        n_k[3] = 0
        klist[3] = khead//"2"
        if( access( klist[3] ) == 1 )
          delete( klist[3], ver- )

        for( j=1; j<=n_image; j+=1 )
        {
          fn = fn_stack[j]
	  fnobe = fn//"bn.e"
	  fnobo = fn//"bn.o"

	  if( access( fnobe//".fits" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fnobe )
	    printf( "Not found '%s'. Skip.\n", fnobe, >> logfn )
	    next
	  }
	  if( access( fnobo//".fits" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fnobo )
	    printf( "Not found '%s'. Skip.\n", fnobo, >> logfn )
	    next
	  }

          imgets( fnobe, 'POS-HWP' )
          theta = real( imgets.value )
          theta_n = int( theta / 22.499999 )

          hwppos = theta_n % 4
	  fullkappa = kappahead//hwppos

          if( hwppos == 0 )
          {
	    n_q += 1
	    buf = "00"//str( n_q )
	    tmp_fn = trilist[i]//".q."//\
	             substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fn, fullkappa,\
	            tmp_fn, >> qlist )

	    n_k[1] = n_k[1] + 1
            buf = "00"//str( n_k[1] )
            tmp_fn = trilist[i]//".kap.0."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    tmp_fn1 = klist[1]
            printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
          }

          if( hwppos == 2 )
          {
	    n_q += 1
	    buf = "00"//str( n_q )
	    tmp_fn = trilist[i]//".q."//substr( buf, ( strlen( buf ) - 2 ),\
                    strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fullkappa, fn,\
	            tmp_fn, >> qlist )

	    n_k[3] = n_k[3] + 1
            buf = "00"//str( n_k[3] )
            tmp_fn = trilist[i]//".kap.2."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    tmp_fn1 = klist[3]
            printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
          }
        }
      }

      if( n_u == 0 )
      {
        qumode = "KAPPA"
        printf( "Not found any pair for u. Then execute u for each image.\n" )
        printf( "Not found any pair for u. Then execute u for each image.\n",\
                >> logfn )

        n_k[2] = 0
        klist[2] = khead//"1"
        if( access( klist[2] ) == 1 )
          delete( klist[2], ver- )
        n_k[4] = 0
        klist[4] = khead//"3"
        if( access( klist[4] ) == 1 )
          delete( klist[4], ver- )

        for( j=1; j<=n_image; j+=1 )
        {
          fn = fn_stack[j]
	  fnobe = fn//"bn.e"
	  fnobo = fn//"bn.o"

	  if( access( fnobe//".fits" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fnobe )
	    printf( "Not found '%s'. Skip.\n", fnobe, >> logfn )
	    next
	  }
	  if( access( fnobo//".fits" ) == 0 )
          {
	    printf( "Not found '%s'. Skip.\n", fnobo )
	    printf( "Not found '%s'. Skip.\n", fnobo, >> logfn )
	    next
	  }

          imgets( fnobe, 'POS-HWP' )
          theta = real( imgets.value )
          theta_n = int( theta / 22.499999 )

          hwppos = theta_n % 4
	  fullkappa = kappahead//hwppos

          if( hwppos == 1 )
          {
	    n_u += 1
	    buf = "00"//str( n_u )
	    tmp_fn = trilist[i]//".u."//\
	             substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fn, fullkappa,\
	            tmp_fn, >> ulist )

	    n_k[2] = n_k[2] + 1
            buf = "00"//str( n_k[2] )
            tmp_fn = trilist[i]//".kap.1."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    tmp_fn1 = klist[2]
            printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
          }

          if( hwppos == 3 )
          {
	    n_u += 1
	    buf = "00"//str( n_u )
	    tmp_fn = trilist[i]//".u."//substr( buf, ( strlen( buf ) - 2 ),\
                    strlen( buf ) )
	    printf( "%s  %s %s %s\n", qumode, fullkappa, fn,\
	            tmp_fn, >> ulist )

	    n_k[4] = n_k[4] + 1
            buf = "00"//str( n_k[4] )
            tmp_fn = trilist[i]//".kap.3."//substr( buf, ( strlen( buf ) - 2 ), strlen( buf ) )
	    tmp_fn1 = klist[4]
            printf( "%s  %s\n", fn, tmp_fn, >> tmp_fn1 )
          }
        }
      }

#----- print results and make file list

      printf( "---- I images (%d sets)  %s  ---\n", n_i, ilist )
      printf( "---- I images (%d sets)  %s  ---\n", n_i, ilist, >> logfn )
      type( ilist )
      type( ilist, >> logfn )
      printf( "---- Q/I group (%d sets)  %s  ---\n", n_q, qlist )
      printf( "---- Q/I group (%d sets)  %s  ---\n", n_q, qlist, >> logfn )
      type( qlist )
      type( qlist, >> logfn )
      printf( "---- U/I group (%d sets)  %s  ---\n", n_u, ulist )
      printf( "---- U/I group (%d sets)  %s  ---\n", n_u, ulist, >> logfn )
      type( ulist )
      type( ulist, >> logfn )
      printf( "\n" )
      printf( "\n", >> logfn )

      printf( "%s %s %s %s %s %s %d %s\n", trilist[i], ilist, qlist, ulist,\
              diaph[i], filter[i], n_theta_n, khead, >> iqulist )
      printf( "%s %s %s %s %s %s %d %s\n", trilist[i], ilist, qlist, ulist,\
              diaph[i], filter[i], n_theta_n, khead, >> logfn )
    }
  }

end
