#
#       TRISPEC REDUCTION SUB ROUTINE
#
#       Define parameters for each OBJ group : triparam.cl
#
#       2006/12/18      K.Kawabata  Ver.0.50
#       2009/02/15      K.Kawabata  Ver.0.51 Adding sky-subtraction method
#
procedure triparam

begin

  int i, j, c, d
  string trilist[500]
  int trilistnum[500]
  string shorttyp[500], title[500]
  string exptime[500], diaph[500], filter[500], slit[500]
  string prmfile, tmp_list
  string listoflist, buf=""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int n_list
  file caldir, fn_def, etcdir
  file logfn
  string fn_head
  string calmode = "" # 1:object, 2:UP,SP,GT

  string binmode # 1:bin 2:mean

  string featmode # 1:auto(7600A) 2:manual

  string binsel, binfn, binhead, binewfn
  real consterr
  string refbin
  string bfn_stack[100]
  string bfn_tmp
  int n_bfn
  real binc, binw, binw_tmp
  int nx2
  real crpix, crval, cdelt

  file tmp_fn
  string dateobs

  string kapmode

  real rsigma

  int flag_all = 0
  real consterr_all
  string binsel_all


#--- initial settings

  listoflist = envget( "_rootlist" )
  caldir = envget( "_caldir" )
  etcdir = envget( "_etcdir" )
  fn_def = caldir//envget( "_defhname" )
  binhead = envget( "_bintable" )
  logfn = envget( "_logfile" )

  calmode = envget ( "_objcal" )
  if( calmode == "" ) # for old version of triinit.cl
    calmode = "1"

  if( access( "tri.tmp.time" ) == 1 )
    delete( "tri.tmp.time", ver- )
  time >> tri.tmp.time
  list = "tri.tmp.time"
  if( fscan( list, buf, buf1, buf2 ) != 3 )
  {
    printf( "Error: time task -- reading\n" )
    bye
  }
  delete( "tri.tmp.time", ver- )

  tmp_list = mktemp( "tri.tmp.list." )
  tmp_fn = mktemp( "tri.tmp.fn." )

  printf( "\n" )
  printf( "%s %s\n", buf1, buf2 )
  printf( "--- Parameter Setting for Each Group\n" )
  printf( "\n" )
 
  printf( "\n", >> logfn )
  printf( "%s %s\n", buf1, buf2, >> logfn )
  printf( "--- Parameter Setting for Each Group\n", >> logfn )
  printf( "\n", >> logfn )

#--- check list file

  if( access( listoflist ) == 0 )
  {
    if( access( listoflist//".orig" ) == 1 )
    {
      copy( listoflist//".orig", listoflist )
    }
    else
    {
      print( "Cannot find "//listoflist//"   Please do trigroup" )
      print( "Cannot find "//listoflist//"   Please do trigroup", >> logfn )
      bye
    }
  }
  if( access( listoflist//".orig" ) == 0 )
    copy( listoflist, listoflist//".orig" )

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    printf( "Cannot find '%s'\n", fn_def, >> logfn )
    bye
  }

  crpix = 1.0
  crval = 0.0
  cdelt = 0.0

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
    {
      crval = real( buf3 )
    }
    if( buf2 == "DW" )
    {
      cdelt = real( buf3 )
    }
    if( buf2 == "NX2" )
    {
      nx2 = int( buf3 )
    }
  }
  if( ( crval * cdelt ) == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    printf( "Read error in '%s'\n", fn_def, >> logfn )
    bye
  }

#----- Calibration mode

  if( calmode == "2" )
  {
    printf( "Entering calibration data mode...\n" )
    printf( "[2a]:No bin  [2b]:assigned bin [1]: object mode >> " )
    printf( "Entering calibration data mode...\n", >> logfn )
    printf( "[2a]:No bin  [2b]:assigned bin [1]: object mode >> ", >> logfn )
    c = scan( calmode )
    while( calmode != "2a" && calmode != "2b" && calmode != "1" )
      c = scan( calmode )
    printf( "%s\n", calmode, >> logfn )
  }

#----- Scanning the list file

  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL" )
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL",\
       >> logfn )
      break
    }
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7

    i = strlen( trilist[n_list] )
    while( substr( trilist[n_list], i, i ) != "." )
      i -= 1

    trilistnum[n_list] = int( substr( trilist[n_list], ( i + 1 ), strlen( trilist[n_list] ) ) )
  }


#----- Main routine

  for( i=1; i<=n_list; i+=1 )
  {
    if( shorttyp[i] == "OBJ" )
    {
      prmfile = trilist[i]//".prm"
      printf( "--- OBJECT %s  %s %s %s ---\n", trilist[i], diaph[i],\
              exptime[i], filter[i] )
      printf( "--- OBJECT %s  %s %s %s ---\n", trilist[i], diaph[i],\
              exptime[i], filter[i], >> logfn )
      consterr = 0.0
      refbin = "---"
      
      if( calmode== "1" )
      {
        if( access( prmfile ) == 1 )
        {
          printf( "Old parameter file detected:\n" )
          printf( "Old parameter file detected:\n", >> logfn )
          type( prmfile )
	  printf( "Modify it? (y/n) >> " )
	  printf( "Modify it? (y/n) >> ", >> logfn )
          c = scan( buf )
	  while( buf != "y" && buf != "n" )
            c = scan( buf )
	  printf( "%s\n", buf, >> logfn )
          if( buf == "n" )
	    next
	  delete( prmfile, ver- )
	}
      }
      else
      {
        if( access( prmfile ) == 1 )
	  delete( prmfile, ver- )
      }

  #---- binning mode

    selectbtype:

      binmode = "1"
      binfn = trilist[i]//".bintab"
      binewfn = trilist[i]//".binew"
      if( access( binewfn ) == 1 )
      {
        delete( binewfn, ver- )
      }	
      if( access( binfn ) == 1 )
      {
        delete( binfn, ver- )
      }
      buf = ""
      binsel = ""
      if( calmode == "2b" )
      {
        if( flag_all == 0 )
	{
          printf( "Binning mode [1]:const error [4]:def.file >> " )
          printf( "Binning mode [1]:const error [4]:def.file >> ",\
	  >> logfn )
          c = scan( binsel )
          while( binsel != "1" && binsel != "4" )
	  {
	    binsel = ""
            c = scan( binsel )
          }
	  printf( "%s\n", binsel, >> logfn )
	  binsel_all = binsel
	
          if( binsel == "1" )
	  {
            printf( "Constant error setting in percent [0.01-1.0] >> " )
            printf( "Constant error setting in percent [0.01-1.0] >> ",\
                    >> logfn )
            c = scan( consterr )
	    printf( "%4.2f\n", consterr, >> logfn )
	    consterr_all = consterr
	  }
	  else
	  {
            printf( "This function is not working now... Sorry.\n" )
            printf( "This function is not working now... Sorry.\n", >> logfn )
	    bye
	  }
	  flag_all = 1
        }
	else
	{
          binsel = binsel_all
	
          if( binsel == "1" )
	  {
	    consterr = consterr_all
	  }
	  else
	  {
            printf( "This function is not working now... Sorry.\n" )
            printf( "This function is not working now... Sorry.\n", >> logfn )
	    bye
	  }
        }
      }	
      if( calmode == "1"  )
      {
        printf( "Binning mode [RET]:org [1]:const error [2]:sum [3]:mean [4]:def.file >> " )
        printf( "Binning mode [RET]:org [1]:const error [2]:sum [3]:mean [4]:def.file >> ",\
	>> logfn )
        c = scan( binsel )
        while( binsel != "" && binsel != "1" && binsel != "2" && \
	       binsel != "3" && binsel != "4" )
	{
	  binsel = ""
          c = scan( binsel )
        }
	printf( "%s\n", buf, >> logfn )
      }
      if( binsel == "" )
      {
        binsel = "0"
        for( j=1; j<=nx2; j+=1 )
	{
          binc = crval + ( j - crpix ) * cdelt
          binw = cdelt
	  printf( "%9.4f  %10.5f\n", binc, binw, >> binfn )
        }
      }
      if( calmode == "1" && binsel == "1" )
      {
        buf = ""
        printf( "[1]: Constant error  [2]: Use same bin-table as others >> " )
        c = scan( buf )
	while( buf != "1" && buf != "2" )
          c = scan( buf )
	if( buf == "1" )
	{
    consterrinput:
          printf( "Constant error setting in percent [0.01-1.0] >> " )
          printf( "Constant error setting in percent [0.01-1.0] >> ",\
	            >> logfn )
          c = scan( consterr )
          if( consterr < 0.01 )
          {
	    printf( "Too small. Input again.\n" )
	    printf( "Too small. Input again.\n", >> logfn )
	    goto consterrinput
	  }
	  if( consterr > 1.00 )
	  {
	    printf( "Too large. Input again.\n" )
	    printf( "Too large. Input again.\n", >> logfn )
	    goto consterrinput
	  }
	  printf( "%4.2f\n", consterr, >> logfn ) 
        }
	else
	{
          for( j=1; j<=n_list; j+=1 )
          {
            if( shorttyp[j] == "OBJ" )
            {
	      printf( "%3d: %s\n", j, trilist[j] )
	      printf( "%3d: %s\n", j, trilist[j], >> logfn )
	    }
	  }
	  printf( "---\n" )
	  printf( "Select group number >> " )
	  printf( "---\n", >> logfn )
	  printf( "Select group number >> ", >> logfn )
	inputrelgr:
	  c = scan( d )
	  while( d < 0 || d > n_list )
            c = scan( d )
	  if( shorttyp[d] != "OBJ" )
	    goto inputrelgr
	  refbin = trilist[d]//".bintab"
	}
      }
      if( binsel == "2" )
      {
    binfacinput:
        printf( "Binning factor in angstrom (>%5.3f) >> ", cdelt )
        printf( "Binning factor in angstrom (>%5.3f) >> ", cdelt, >> logfn )
        c = scan( binw_tmp )
	if( binw_tmp < cdelt )
	{
	  printf( "Too small. Input again.\n" )
	  printf( "Too small. Input again.\n", >> logfn )
	  goto binfacinput
	}
        for( j=1; j<=nx2; j+=1 )
	{
          binc = crval + ( j - crpix ) * binw_tmp
          binw = binw_tmp
	  if( j == 1 )
	  {
	    binc = binc + 0.5*binw;
	    binw = 0.5*binw;
	  }
	  if( ( binc + 0.5*binw ) > ( crval + ( nx2 - crpix ) * cdelt ) )
	  {
	    binc = 0.5*( (binc-0.5*binw) + ( crval + ( nx2 - crpix ) * cdelt ) )
	    binw = ( ( crval + ( nx2 - crpix ) * cdelt ) - binc ) * 2.0
	  }
	  if( binc > ( crval + ( nx2 - crpix ) * cdelt ) )
	    break
	  printf( "%9.4f  %10.5f\n", binc, binw, >> binfn )
	}
      }
      if( binsel == "3" )
      {
        binmode = "2"
        printf( "Running-mean factor in angstrom (>%5.3f) >> ", cdelt )
        printf( "Running-mean factor in angstrom (>%5.3f) >> ", cdelt,\
	 >> logfn )
        c = scan( binw_tmp )
	if( binw_tmp < cdelt )
	{
	  printf( "Too small. Input again.\n" )
	  printf( "Too small. Input again.\n", >> logfn )
	  goto binfacinput
	}
        for( j=1; j<=nx2; j+=1 )
	{
          binc = crval + ( j - crpix ) * cdelt
          binw = binw_tmp
	  if( ( binc - 0.5*binw ) < crval )
	  {
	    binc = 0.5*( crval + (binc+0.5*binw) )
	    binw = ( binc - crval ) * 2.0
	  }
	  if( ( binc + 0.5*binw ) > ( crval + ( nx2 - crpix ) * cdelt ) )
	  {
	    binc = 0.5*( (binc-0.5*binw) + ( crval + ( nx2 - crpix ) * cdelt ) )
	    binw = ( ( crval + ( nx2 - crpix ) * cdelt ) - binc ) * 2.0
	  }
	  printf( "%9.4f  %10.5f\n", binc, binw, >> binfn )
	}
      }
      if( binsel == "4" )
      {
        n_bfn = 0
        chdir _etcdir$
	sections( binhead, option="fullname", >> tmp_list )
	list = tmp_list
	while( fscan( list, buf ) == 1 )
	{
	  n_bfn += 1
	  bfn_stack[n_bfn] = buf
	}
	delete( tmp_list, ver- )
	back
	if( n_bfn == 0 )
	{
	  printf( "Binning factor file '%s' does not found in %s.\n",\
	          binhead, etcdir )
	  printf( "Binning factor file '%s' does not found in %s.\n",\
	          binhead, etcdir, >> logfn )
	  goto selectbtype
	}
	printf( "These files detected in %s...\n", etcdir )
	printf( "These files detected in %s...\n", etcdir, >> logfn )
	for( k=1; k<=n_bfn; k+=1 )
	{
	  printf( "%2d  %s\n", k, bfn_stack[k] )
	}
	printf( "Select feature table [1-%d] >> ", n_bfn )
	printf( "Select feature table [1-%d] >> ", n_bfn, >> logfn )
	while( scan( c ) != 1 ){}
	bfn_tmp = bfn_stack[c]
	copy( etcdir//bfn_tmp, binfn, ver+ )
        printf( "%d  %s\n", c, bfn_tmp, >> logfn )
#        printf( "%d  %s\n", c, binfn, >> logfn )

#        printf( "Select mode [1]:sum [2]:mean >> " )
#        c = scan( buf )
#        if( buf != "1" && buf != "2" )
#          c = scan( buf )
#        binmode = buf
        refbin = binfn
        binmode = "1"
      }
      if( access( binfn ) == 0 && binsel != "1" )
      {
	printf( "Binning factor file '%s' not found. Something was wrong\n",\
	 binfn )
	printf( "Binning factor file '%s' not found. Something was wrong\n",\
	 binfn, >> logfn )
	goto selectbtype
      }

    #----- Kappa mode / second order light mode / cross-talk mode

      if( calmode == "1" )
      {
	kapmode = "222"
	printf( "[CR reduction 1:ON 2:OFF][2nd_ord cor. 1:ON 2:OFF][C.talk cor. 1:ON 2:OFF]\n" )
        printf( "Select sp. calc. mode with three numbers. ([RET]: %s) >> ",\
                 kapmode )
	printf( "[CR reduction 1:ON 2:OFF][2nd_ord cor. 1:ON 2:OFF][C.talk cor. 1:ON 2:OFF]\n", >> logfn )
        printf( "Select sp. calc. mode with three numbers. ([RET]: %s) >> ",\
                 kapmode, >> logfn )
        c = scan( kapmode )
        while( kapmode != "111" && kapmode != "112" &&\
	       kapmode != "121" && kapmode != "122" &&\
	       kapmode != "211" && kapmode != "212" &&\
               kapmode != "221" && kapmode != "222" )
          c = scan( kapmode )
	printf( "%s\n", kapmode, >> logfn )
      }
      else
      {
        if( substr( title[i], 1, 1 ) == "G" )
	{
#          kapmode = "111"
          kapmode = "222"
	}
	else
	{
          kapmode = "222"
	}
      }

    #----- assign filename

      if( calmode == "1" )
      {
        printf( "Root-name of output table (NONE: for no output)>> " )
        printf( "Root-name of output table (NONE: for no output)>> ",\
	 >> logfn )
        c = scan( buf )
        while( c == 0 || strlen( buf ) < 1 )
          c = scan( buf )
        fn_head = buf
	printf( "%s\n", buf, >> logfn )
      }
      else
      {
	list = trilist[i]
	c = fscan( list, tmp_fn )
	c = fscan( list, tmp_fn )
	c = fscan( list, tmp_fn )
	if( access( tmp_fn ) == 0 )
	{
	  c = fscan( list, tmp_fn )
	  if( access( tmp_fn ) == 0 )
	  {
	    if( access( tmp_list ) == 1 )
	      delete( tmp_list, ver- )
            sed( 's/fits//', trilist[i], >> tmp_list )
	    list = tmp_list
	    c = fscan( list, tmp_fn )
	    c = fscan( list, tmp_fn )
	    c = fscan( list, tmp_fn )
	    if( access( tmp_fn//"od.e.fits" ) == 1 )
	    {
	      tmp_fn = tmp_fn//"od.e"
	      delete( tmp_list, ver- )
	    }
	    else
	    {
	      c = fscan( list, tmp_fn )
	      if( access( tmp_fn//"od.e.fits" ) == 1 )
	      {
	        tmp_fn = tmp_fn//"od.e"
	        delete( tmp_list, ver- )
	      }
	      else
	      {
	        delete( tmp_list, ver- )
	        printf( "Cannot find a few images in '%s'.\n", trilist[i] )
	        printf( "Cannot find a few images in '%s'.\n",\
		 trilist[i], >> logfn )
		tmp_fn = ""
	      }
	    }
	  }
	}
	if( tmp_fn != "" )
	{
	  imgets( tmp_fn, "DATE-OBS" )
	  buf = imgets.value
	  if( substr( buf, 3, 3 ) == "/" )
	    dateobs = substr( buf, 7, 8 )//substr( buf, 4, 5 )//substr( buf, 1, 2 )
	  else
	    dateobs = substr( buf, 3, 4 )//substr( buf, 6, 7 )//substr( buf, 9, 10 )
	}
	else
	{
	  printf( "Input OBS-DATE of '%s' in yymmdd (e.g. 991231) >> ",\
	  trilist[i] )
	  printf( "Input OBS-DATE of '%s' in yymmdd (e.g. 991231) >> ",\
	  trilist[i], >> logfn )
	  c = scan( dateobs )
	  printf( "%s\n", dateobs, >> logfn )
	}
	buf1 = substr( title[i], 1, 1 )
	if( buf1 == "U" )
	  buf1 = "u"
	if( buf1 == "G" )
	  buf1 = "g"
	if( buf1 == "S" )
	  buf1 = "s"
        fn_head = buf1//substr( title[i], 4, 6 )//"_"//dateobs//substr( "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ", trilistnum[i], trilistnum[i] )
#         fn_head = buf1//substr( title[i], 4, 6 )//substr( "abcdefghijklmnopqrstuvwxyz", trilistnum[i], trilistnum[i] )
      }


    #----- Clipping sigma

      if( calmode == "1" )
      {
        printf( "Clipping Sigma for %s  >> ", fn_head )
        printf( "Clipping Sigma for %s  >> ", fn_head, >> logfn )
        c = scan( rsigma )
        while( rsigma <= 0.0 )
          c = scan( rsigma )
	printf( "%f\n", rsigma, >> logfn )
      }
      else
      {
        rsigma = 2.5
      }

    #----- Identify mode at TRIARRANGE

      if( calmode == "1" )
      {
        featmode = "0"
        printf( "Feature identification [1]:auto (7620A) [2]:manual  >> " )
        printf( "Feature identification [1]:auto (7620A) [2]:manual  >> ", >> logfn )
        c = scan( featmode )
        while( featmode != "1" && featmode != "2" )
          c = scan( featmode )
	printf( "%s\n", featmode, >> logfn )
      }
      else
      {
        featmode = "1"
      }

#--- print parameter-file ---      
      
      printf( "%s %6.3f %20s %s %s %s %5.2f %s\n", binsel, consterr, refbin,\
              binmode, kapmode, fn_head, rsigma, featmode, >> prmfile )
      if( calmode == "2" )
      {
        printf( "%s %6.3f %20s %s %s %s %5.2f %s\n", binsel, consterr, refbin,\
	      binmode, kapmode, fn_head, rsigma, featmode )
        printf( "%s %6.3f %20s %s %s %s %5.2f %s\n", binsel, consterr, refbin,\
	      binmode, kapmode, fn_head, rsigma, featmode, >> logfn )
      }

      printf( "\n" )
    }
  }

#----- Rearrange according to the binning-refference 

  for( i=n_list; i>=1; i-=1 )
  {
    if( shorttyp[i] == "OBJ" )
    {
      prmfile = trilist[i]//".prm"
      list = prmfile
      if( fscan( list, binsel, buf, refbin, buf, buf, buf, buf ) != 7 )
      {
        printf( "Read error in %s. Do triparam again!\n", prmfile )
        printf( "Read error in %s. Do triparam again!\n", prmfile, >> logfn )
	bye
      }
      if( binsel == "1" && refbin != "---" )
      {
        buf1 = trilist[i] 
        buf2 = shorttyp[i]
        buf3 = title[i]   
        buf4 = exptime[i] 
        buf5 = diaph[i]   
        buf6 = filter[i]  
        buf7 = slit[i]
	 
	for( j=i; j<=(n_list-1); j+=1 )
	{
          trilist[j]  = trilist[j+1] 
          shorttyp[j] = shorttyp[j+1]
          title[j]    = title[j+1]   
          exptime[j]  = exptime[j+1] 
          diaph[j]    = diaph[j+1]   
          filter[j]   = filter[j+1]  
          slit[j]     = slit[j+1]    
	}
        trilist[n_list]  = buf1
        shorttyp[n_list] = buf2
        title[n_list]    = buf3
        exptime[n_list]  = buf4
        diaph[n_list]    = buf5
        filter[n_list]   = buf6
        slit[n_list]     = buf7
      }
    }
  }
  if( access( listoflist ) == 1 )
    delete( listoflist, ver- )
  
  for( i=1; i<=n_list; i+=1 )
  {
     print( trilist[i]//"   "//shorttyp[i]//" "//title[i]//" "//exptime[i]//\
     " "//diaph[i]//" "//filter[i]//" "//slit[i], >> listoflist )
  }
  
end
