#
#       TRI REDUCTION SUB ROUTINE
#
#       Spectrophotometric correction: triphcor.cl
#
#       1998/07/31      K.Kawabata  Ver.2.10
#

procedure triphcor


begin

  file phlog
  file tmp_list1, tmp_list2
  file tmp_fn1, tmp_fn2
  file fn_conv, fn_extn, fn_refspec
  int c, i, j, n, n_list
  string fnstack[200]
  file fn1, fn2, fn3
  real am, exptime, jd

  real lmd
  real mag

  string buf, buf1, buf2, buf3
  real cdelt, crval, crpix
  int npix
  file fn_def

  int flag_ver       # 0:no display 1:display

#--- initial settings

  fn_conv = "TRI.phot.convf"
  fn_extn = "TRI.phot.extn"
  fn_refspec = "TRI.phot.refspec"

  tmp_list1 = mktemp( "tri.tmp.fn." )
  tmp_list2 = mktemp( "tri.tmp.fn." )
  tmp_fn1 = mktemp( "tri.tmp.image." )
  tmp_fn2 = mktemp( "tri.tmp.image." )
  fn_def = envget( "_caldir" )//envget( "_defhname" )
  phlog = envget( "_phlog" )

  print( "" )
  print( "------------------------------------------" )
  print( " Flux calibration with derived parameters"
  print( "------------------------------------------" )
  print( "" )

  if( access( phlog ) == 1 )
    delete( phlog, ver- )

  if( access( tmp_list1 ) == 1 )
    delete( tmp_list1, ver- )
  if( access( tmp_list2 ) == 1 )
    delete( tmp_list2, ver- )
  if( access( tmp_fn1//".fits" ) == 1 )
    imdelete( tmp_fn1, ver- )
  if( access( tmp_fn2//".fits" ) == 1 )
    imdelete( tmp_fn2, ver- )

  if( access( fn_conv//".fits" ) == 0 && access( fn_conv ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_conv )
    bye
  }
  if( access( fn_extn//".fits" ) == 0 && access( fn_extn ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_extn )
    bye
  }
  if( access( fn_refspec//".fits" ) == 0 && access( fn_refspec ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_refspec )
    bye
  }

#----- read difinition file

  buf1 = ""
  buf2 = ""
  buf3 = ""

  if( access( fn_def ) == 0 )
  {
    printf( "Cannot find '%s'\n", fn_def )
    bye
  }

  crpix = 1.0
  crval = 0.0
  cdelt = 0.0
  npix = 0

  list = fn_def
  while( fscan( list, buf1, buf2, buf3 ) != EOF )
  {
    if( buf2 == "SW" )
    {
      crval = real( buf3 )
   }
    if( buf2 == "DW" )
    {
      cdelt = real( buf3 )
    }
    if( buf2 == "NX3" )
    {
      npix = int( buf3 )
    }
    if( buf2 == "GEADU" )
    {
      geadu = real( buf3 )
    }
    if( buf2 == "NX2" )
    {
      npix = int( buf3 )
    }
  }
  if( ( crval * cdelt * npix ) == 0 )
  {
    printf( "Read error in '%s'\n", fn_def )
    bye
  }

#--- Verbose Mode

  if( envget( "_fg_ver" ) == "1" )
  {
    flag_ver = 1
  }
  else
  {
    if( envget( "_fg_ver" ) == "2" )
    {
      flag_ver = 0
    }
    else
    {
      printf( "'triinit' process is incomplete.(_fg_ver) Retry.\n" )
      bye
    }
  }

#----- Scanning the list file

  sections( "*i2.ave.fits", option="fullname", > tmp_list1 )
  if( sections.nimages == 0 )
  {
    printf( "No I2-spectral image found in this directory.\n" )
    delete( tmp_list1, ver- )
    bye
  }
  sed( 's/i2\.ave\.fits//', tmp_list1, >> tmp_list2 )
  delete( tmp_list1, ver- )

  n_list = 0
  list = tmp_list2
  while( fscan(list, buf1) != EOF )
  {
    n_list += 1
    fnstack[n_list] = buf1
  }
  delete( tmp_list2, ver- )

#----- main routine

  for( i=1; i<=n_list; i+=1 )
  {
    fn1 = fnstack[i]//"i2.ave"
    fn2 = fnstack[i]//"i3.ave"
    fn3 = fnstack[i]//"2.xy"

    if( access( fn1//".fits" ) == 0 )
    {
      printf( "'%s' not found. Skip.\n", fn1 )
      next
    }
    if( access( fn2//".fits" ) == 1 )
      imdelete( fn2, ver- )
    if( access( fn3 ) == 1 )
      delete( fn3, ver- )


    if( flag_ver == 1 )
      printf( "%17s --> %17s, %15s \n", fn1, fn2, fn3 )

    imgets( fn1, "AIRMASS" )
    am = real( imgets.value )
    imgets( fn1, "EXPTIME" )
    exptime = real( imgets.value )
    imgets( fn1, "JD" )
    jd = real( imgets.value )

    imarith( fn1, "/", fn_refspec, tmp_fn1, pixtype="double",\
             calctype="double", ver- )

    imarith( tmp_fn1, "/", exptime, tmp_fn1, pixtype="double",\
             calctype="double", ver- )
    imarith( tmp_fn1, "*", fn_conv, tmp_fn1, pixtype="double",\
             calctype="double", ver- )

    imarith( fn_extn, "*", -am, tmp_fn2, pixtype="double",\
             calctype="double", ver- )
    imfunc( tmp_fn2, tmp_fn2, "aln", ver- )
    imarith( tmp_fn1, "/", tmp_fn2, fn2, pixtype="double",\
             calctype="double", ver- )

    imdelete( tmp_fn1, ver- )
    imdelete( tmp_fn2, ver- )

  # Hz --> AB-magnitude
    imfunc( fn2, fn2, "log10", ver- )
    imarith( fn2, "*", -2.5, fn2, pixtype="double",\
             calctype="double", ver- )
    imarith( fn2, "-", 48.60, fn2, pixtype="double",\
             calctype="double", ver- )

    if( access( tmp_list1 ) == 1 )
      delete( tmp_list1, ver- )
    listpix( fn2, >> tmp_list1 )
    if( access( tmp_list1 ) == 0 )
    {
      printf( "Error in listpix of %s.\n", fn2 )
      bye
    }
    list = tmp_list1
    for( j=1; j<=npix; j+=1 )
    {
      if( fscan( list, buf, buf, buf1 ) != 3 )
      {
        printf( "Error in reading of listpix for %s.\n", fn2 )
        bye
      }
      mag = real( buf1 )
      lmd = crval + ( j - crpix ) * cdelt
      printf( "%7.2f  %6.3f\n", lmd, mag, >> fn3 )
    }
    delete( tmp_list1, ver- )
    printf( "%13s  %13.6s %6.1s %6.3s\n", fn3, jd, exptime, am, >> phlog )
  }
  if( flag_ver == 1 )
    printf( "Log has written in %s.\n", phlog )

end
