#######################################################
#         TRISPEC REDUCTION PIPELINE on IRAF          #
#                                                     #
#              main program: trired.cl                #
#                                                     #
#            2006/11/06  Ver.0.50   K.Kawabata        #
#######################################################

procedure trired

begin

#----- const variables definision

  file histfile = "TRI.history"
  string procname[17]
  string initialproc = "triinit"
  int n_proc

#----- volatile variables definition

  int procid
  int procid0
  int procid1
  int c, last_c, i
  string buf = ""
  string buf1 = ""
  string buf2 = ""
  string buf3 = ""
  int flag_hist = 0

#----- constant variable definitions

  procname[1] = "trifconv"
  procname[2] = "trigroup"
  procname[3] = "tridarkave"
  procname[4] = "tridarksub"
  procname[5] = "tritrim"
  procname[6] = "tribadpix"
  procname[7] = "tripixsens"
  procname[8] = "triextract"
  procname[9] = "triskysub"
  procname[10] = "triarrange"
  procname[11] = "triflat2"
  procname[12] = "tri1d"
  procname[13] = "tribin"
  procname[14] = "triiqugroup"
  procname[15] = "triiqumake"
  procname[16] = "triiquave"
  procname[17] = "trimktable"
  n_proc = 17

  procid = 1
  procid0 = 1
  procid1 = n_proc

#----- DELETE OLD TEMPORALY FILES

  printf( "Now erasing tempolary files...  Just a minute.\n" )
  imdelete('*tmp*.imh')
  delete('*tmp*')

#----- TITLE

  printf( "\n" )
  printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
  printf( "{   TRISPEC REDUCTION TOOL  }\n" )
  printf( "{                           }\n" )
  printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"  )
  printf( "\n" )

#----- CHECK INITIALIZATION SCRIPT

  if( envget( "_initproc" ) != "DONE" )
  {
    printf( "Error: Not Done Initialization.\n" )
    printf( "Please do '%s'\n", initproc )
    bye
  }


#---- Read history file

  if( access( histfile ) == 1 )
  {
    printf( "\nHistory file detected.\n" )
    list = histfile
    c = fscan( list, buf1, procid0, buf2, buf3 )
    last_c = c
    while( c != EOF )
    {
      last_c = c
      c = fscan( list, buf1, procid0, buf2, buf3 )
    }
    if( last_c < 4 )
    {
      buf3 = "ABNORMAL_TERMINATION"
      printf( "%s\n", buf3, >> histfile )
    }

    if( procid0 < 1 || procid0 > n_proc )
    {
      printf( "Error: ProcID is wrong in %s. Ignored.\n", histfile )
    }
    else
    {
      flag_hist = 1
      printf( "LAST PROCEDURE NAME : %s (procID %d)\n",\
              procname[procid0], procid0 )
      printf( "               DATE : %s\n", buf1 )
      if( buf3 == "NORMAL_END" )
      {
        printf( "             STATUS : NORMAL END\n" )
        procid = procid0 + 1
        if( procid > n_proc )
          procid = n_proc
      }
      else
      {
        printf( "             STATUS : ABNORMAL TERMINATION\n" )
        procid = procid0
      }
    }
  }

#----- Select Procedure

  if( flag_hist == 1 )
    printf( "[RET]:Continue.(from '%s') [1]:Customize >> ",\
            procname[procid] )
  else
    printf( "[RET]:New Reduction [1]:Customize >> " )
  buf = ""
  c = scan( buf )
  while( buf != "" && buf != "1" )
  {
    buf = ""
    c = scan( buf )
  }
  if( buf == "1" )
  {
    for( i=1; i<=n_proc; i+=1 )
      printf( "%2d  %s\n", i, procname[i] )
    printf( "Biginning Proc-ID number >> " )
    c = scan( procid )
    while( procid < 0 || procid > n_proc )
      c = scan( procid )
    printf( "End Proc-ID number >> " )
    c = scan( procid1 )
    while( procid1 < 0 || procid1 > n_proc )
      c = scan( procid1 )
  }

#----- main routine

  while( procid > 0 && procid <= procid1 )
  {
    if( access( "tri.tmp.time" ) == 1 )
      delete( "tri.tmp.time", ver- )
    time >> tri.tmp.time

    list = "tri.tmp.time"
    if( fscan( list, buf, buf1, buf2 ) != 3 )
    {
      printf( "Error: time task/read\n" )
      bye
    }
    delete( "tri.tmp.time", ver- )
    printf( "\n%s-%s  %2d %s\n", buf2, buf1, procid, procname[procid] )
    printf( "%s-%s  %2d %s  ", buf2, buf1, procid, procname[procid],\
            >> histfile )
    
    switch( procid )
    {
      case 1:
        trifconv
        break
      case 2:
        trigroup
        break
      case 3:
        tridarkave
        break
      case 4:
        tridarksub
        break
      case 5:
        tritrim
        break
      case 6:
        tribadpix
        break
      case 7:
        tripixsens
        break
      case 8:
        triextract
        break
      case 9:
        triskysub
        break
      case 10:
        triarrange
        break
      case 11:
        triflat2
        break
      case 12:
        tri1d
        break
      case 13:
        tribin
        break
      case 14:
        triiqugroup
        break
      case 15:
        triiqumake
        break
      case 16:
        triiquave
        break
      case 17:
        trimktab
        break
      default:
        printf( "Procedure ID error !!!  Restart,please.\n" )
        printf( "(If you want re-examine, select manual mode)\n" )
	bye
    }
    printf( "NORMAL_END\n", >> histfile )
    procid += 1

  }

#----- ending

  if( procid > 16 )
  {
    printf( "Minimize disk space? (y/n) >> " )
    buf = ""
    c = scan( buf )
    while( buf != "y" && buf != "n" )
      c = scan( buf )
    if( buf == "y" )
      triclean

#   imdelete('*tmp*.imh')
#    delete('*tmp*')

    printf( "\n" )
    printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
    printf( "{   TRIRED FINISHED.   Let's Enjoy Astronomy.    }\n" )
    printf( "{                                                }\n" )
    printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
  }
  else
  {
    if( procid > 12 )
    {
      printf( "\n" )
      printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
      printf( "{   TRIRED terminated.    Just Over There!       }\n" )
      printf( "{                                                }\n" )
      printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
    }
    else
    {
      printf( "\n" )
      printf( "{~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~}\n" )
      printf( "{   TRIRED terminated.   Long Way to Finish...   }\n" )
      printf( "{                                                }\n" )
      printf( " ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n" )
    }
  }
  bye

end

