#
#       TRI REDUCTION SUB ROUTINE
#
#       shift conserving the total count
#                                               : trishift.cl
#
#       1997/04/15      K.Kawabata  Ver.1.00
#       1997/09/04      K.Kawabata  Ver.1.10  original C prog
#       2000/02/29      K.Kawabata  Ver.3.00  for SITe-CCD
#       2005/04/07      M.Isogai    Ver.3.10  for iraf v2.12
#
#
#    !!! WARNING: File name must not be included '.imh' !!!
#

procedure trishift( infile, outfile, x_shift, y_shift )

string infile { prompt = "Input image" }
string outfile { prompt = "output image" }
real x_shift { prompt = "x shift value" }
real y_shift { prompt = "y shift value" }


begin

  file infn, outfn
  file fn_prefit
  file fn1, fn2, fn3, fn4
  real xs, ys
  string buf1, buf2
  int c
  int n_version      # IRAF version (210 or 211)

  infn = infile
  outfn = outfile
  xs = x_shift
  ys = y_shift

#----- IRAF version check

  buf1 = envget( "version" )
  c = stridx( "V", buf1 )
  buf2 = substr( buf1, c, ( c + 4 ) )

  n_version = 0

  if( buf2 == "V2.11" || buf2 == "V2.12" || buf2 == "V2.13" || buf2 == "V2.14")
  {
    n_version = 211
  }
  else
  {
    if( buf2 == "V2.10" )
    {
      n_version = 210
    }
  }

  if( n_version == 0 )
  {
    printf( "IRAF version check failed.\n" )
    printf( "Please input like next line, and restart 'trired'\n"
    printf( "cl> set version V2.10  (V2.10 and V2.11 are valid)\n"
    bye
  }

#---
  
  if( access( outfn//".fits" ) )
    imdelete( outfn, ver- )

  if( access( infn ) )
    delete( infn, ver- )
  if( access( outfn ) )
    delete( outfn, ver- )

  if( n_version == 210 )
  {
    wfits( infn, infn, bscale=1.0, bzero=0.0, long_header-,
           short_header-, bitpix=-32, scale-, autoscale- )
  }
  else
  {
    if( n_version == 211 )
    {
      wfits( infn//".fits", infn, bscale=1.0, bzero=0.0, long_header-,
             fextn="", short_header-, bitpix=-32, scale-, autoscale- )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }

  copy( infn, outfn, ver- )

  trishiftsub( infn, outfn, xs, ys )

  if( n_version == 210 )
  {
    rfits( outfn, "*", outfn, make_image+, long_header-, short_header-,\
           datatype="real", blank=0., scale-, oldirafname-, offset=0 )
  }
  else
  {
    if( n_version == 211 )
    {
      rfits( outfn, "", outfn//".fits", make_image+, long_header-,\
             short_header-,\
             datatype="real", blank=0., scale-, oldirafname-, offset=0 )
    }
    else
    {
      printf( "IRAF version erorr (%d)\n", n_version )
      bye
    }
  }


  hedit( outfn, "HISTORY", "shifted by ("//xs//", "//ys//")",
         delete-, verify-, show-, update+ )

  delete( infn, ver- )
  delete( outfn, ver- )

end
