#
#       TRISPEC REDUCTION SUB ROUTINE
#
#       trisky1d.cl: Sky subtruction and make 1-d spectra
#
# flat_IR1.0001.fits  

procedure trisky1d( inlist, inflat, sk_y1, sk_y2, sk_y3, sk_y4 )

string inlist { prompt = "Input image list" }
string inflat { prompt = "Flat-field head name" }
int sk_y1 { prompt = "Start y-position of first sky region" }
int sk_y2 { prompt = "End y-position of first sky region" }
int sk_y3 { prompt = "Start y-position of second sky region" }
int sk_y4 { prompt = "End y-position of second sky region" }

begin

  string infn
  string fnhead[300]
  string flathead, flatfn1, flatfn2
  int n_list
  file fn11, fn21, fn31, fn41
  file fn12, fn22, fn32, fn42
  string buf = ""
  file tmp_fn1
  int i
  int y1, y2, dy
  int sky1, sky2, sky3, sky4 

  infn = inlist
  flathead = inflat
  sky1 = sk_y1
  sky2 = sk_y2
  sky3 = sk_y3
  sky4 = sk_y4

  tmp_fn1 = mktemp( "tri.tmp.fn." )//".fits"

  if( access( tmp_fn1 ) == 1 )
    imdelete( tmp_fn1, ver- )

### Read list file ###

  if( access( infn ) )
  {
    printf( "INLIST '%s' found. ", infn )
  }
  else
  {
    while( access( infn ) == 0 )
    {
      printf( "INLIST '%s' not found. Enter file name >> ", infn )
      i = scan( infn )
    }
    printf( "INLIST '%s' found. ", infn )
  }

  sed( 's/fl\.fits//', infn, > tmp_fn1 )

  n_list = 0
  list = tmp_fn1
  while( fscan( list, buf ) != EOF )
  {
    n_list += 1
    fnhead[n_list] = buf
  }
  delete( tmp_fn1, ver- )
  printf( "%d files registed.\n", n_list )

### mail routine ###

  for( i=1; i<=n_list; i+=1 )
  {
    fn11 = fnhead[i]//"tr.0001.fits"
    fn21 = fnhead[i]//"sk.0001.fits"
    fn31 = fnhead[i]//"od.0001.fits"
    fn41 = fnhead[i]//"or.0001.fits"
    fn12 = fnhead[i]//"tr.0002.fits"
    fn22 = fnhead[i]//"sk.0002.fits"
    fn32 = fnhead[i]//"od.0002.fits"
    fn42 = fnhead[i]//"or.0002.fits"

    flatfn1 = flathead//".0001.fits"
    flatfn2 = flathead//".0002.fits"

    printf( "%s %s ", fn11, fn12 )

    if( access( fn11 ) == 0 || access( fn12 ) == 0 )
    {
      printf( "Not found. Skipped.\n" )
      next
    }

    if( access( flatfn1 ) == 0 || access( flatfn2 ) == 0 )
    {
      printf( "Flat (%s or %s) not found. Skipped.\n", flatfn1, flatfn2 )
      next
    }

    if( access( fn21 ) == 1 || access( fn22 ) == 1 )
    {
      printf( "sk image overwrite...  " )
      if( access( fn21 ) == 1 )
        imdelete( fn21, ver- )
      if( access( fn22 ) == 1 )
        imdelete( fn22, ver- )
    }

    if( access( fn31 ) == 1 || access( fn32 ) == 1 )
    {
      printf( "od image overwrite...  " )
      if( access( fn31 ) == 1 )
        imdelete( fn31, ver- )
      if( access( fn32 ) == 1 )
        imdelete( fn32, ver- )
    }
    if( access( fn41 ) == 1 || access( fn42 ) == 1 )
    {
      printf( "or image overwrite...  " )
      if( access( fn41 ) == 1 )
        imdelete( fn41, ver- )
      if( access( fn42 ) == 1 )
        imdelete( fn42, ver- )
    }

    y1 = sky2 + 1
    y2 = sky3 - 1
    dy = y2 - y1 + 1

    background( fn11, fn21, sample=sky1//":"//sky2//","//sky3//":"//sky4,\
                axis=2, interactive-, order=2, naverage=1,\
                low_reject=1.5, high_reject=1.5, niterate=3,
                grow=0 )
    background( fn12, fn22, sample=sky1//":"//sky2//","//sky3//":"//sky4,\
                axis=2, interactive-, order=2, naverage=1,\
                low_reject=1.5, high_reject=1.5, niterate=3,
                grow=0 )

    blkavg( fn21//"[*,"//y1//":"//y2//"]", fn31, 1, dy, option="sum" )
    blkavg( fn22//"[*,"//y1//":"//y2//"]", fn32, 1, dy, option="sum" )

    if( access( tmp_fn1 ) == 1 )
      imdelete( tmp_fn1, ver- )
    imarith( fn21, "/", flatfn1, tmp_fn1, ver- )
    blkavg( tmp_fn1//"[*,"//y1//":"//y2//"]", fn41, 1, dy, option="sum" )
    imdelete( tmp_fn1, ver- )
    imarith( fn22, "/", flatfn2, tmp_fn1, ver- )
    blkavg( tmp_fn1//"[*,"//y1//":"//y2//"]", fn42, 1, dy, option="sum" )
    imdelete( tmp_fn1, ver- )

    printf( "\n" )
  }
  
  printf( "\n" )

end
