#
#       TRI REDUCTION SUB ROUTINE
#
#       Averaging the difference between celetial position angle and
#       instrumental one in the SP standard tables :  trispave.cl
#
#       1997/09/10      K.Kawabata ver.1.00
#       2000/02/29      K.Kawabata ver.3.00
#
#
procedure trispave( ref_dir )

file ref_dir {prompt = "Directory of UP/GT correction table"}

begin

#  string lspfileid = "s*l*0.tmp1"
#  string uspfileid = "s*u*0.tmp1"
  string lspfileid = "*0.ins"
  string uspfileid = "*0.ins"

  file fn_spdat0 = "spcat.dat"
  file fn_spdat

  file fn_lave = "spavel.xy"
  file fn_uave = "spaveu.xy"
  string gthead, sphead, uphead, diaph
  string lfile = "spave.log"
  string ltmp = "spave.l.tmp"
  string utmp = "spave.u.tmp"
  file avelist

  file tmp_list, tmp_fn
  int n_list
  string fn[200], shdiaph[200], cmnt[200]
  string fn_sp
  real pa_cel, pa_ins, pa_diff
  real sw = 5300.0
  real ew = 5800.0
  real pa_diff_ave
  int n
  
  string rdir, etcdir
  string gtlfile, gtufile, uplfile, upufile
  string buf=""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  int c, i
  int flag_lsel, flag_usel

  print( "" )
  print( "---------------------------------------------------" )
  print( " Averaging instrumental rotation of position angle"
  print( "---------------------------------------------------" )
  print( "" )


  sphead = envget( "_sphead" )
  gthead = envget( "_gthead" )
  uphead = envget( "_uphead" )
  avelist = envget( "_avelog" )

  rdir = ref_dir
  etcdir = envget( "_etcdir" )
  tmp_list = mktemp( "tri.tmp.list." )
  tmp_fn = mktemp( "tri.tmp.file." )

  fn_spdat = etcdir//fn_spdat0

#----- check average log

  if( access( avelist ) == 0 )
  {
    printf( "'%s' not found.\n", avelist )
    bye
  }

#----- check SP data

  if( access( fn_spdat ) == 0 )
  {
    printf( "SP data list: '%s' not found.\n", fn_spdat )
    bye
  }

#----- logfile

  if( access( ltmp ) == 1 )
    delete( ltmp, ver- )
  if( access( utmp ) == 1 )
    delete( utmp, ver- )

  if( access( lfile ) == 1 )
  {
    printf( "Logfile detected. [1]:appand [2]:overwrite  >> " )
    c = scan( buf )
    while( buf != "1" && buf != "2" )
      c = scan( buf )
    if( buf == "2" )
      delete( lfile, ver- )
  }

#----- select diaphragm

  printf( "Select Diaphragm  [1]:2HL [2]:2S2  >> " )
  
  c = scan( buf )
  while( buf != "1" && buf != "2" )
    c = scan( buf )
  if( buf == "1" )
    diaph = "2HL"
  if( buf == "2" )
    diaph = "2S2"

  fn_lave = sphead//diaph//"L"
  fn_uave = sphead//diaph//"U"

  gtlfile = rdir//"/"//gthead//diaph//"L"
  gtufile = rdir//"/"//gthead//diaph//"U"
  uplfile = rdir//"/"//uphead//diaph//"L"
  upufile = rdir//"/"//uphead//diaph//"U"

  if( access( fn_lave ) )
    delete( fn_lave, ver- )
  if( access( fn_uave ) )
    delete( fn_uave, ver- )

#----- make log file

lowerhole:

#----- lower hole

  flag_lsel = 0
  sections( lspfileid, option="fullname", > tmp_list )
  n_list = sections.nimage
  if( n_list == 0 )
  {
    printf( "Cannot find any table. Skip lower hole.\n" )
    delete( tmp_list, ver- )
    goto upperhole
  }

  i = 0
  list = tmp_list
  while( fscan( list, buf ) != EOF )
  {
    i += 1
    fn[i] = buf
  }
  delete( tmp_list, ver- )

  for( i=1; i<=n_list; i+=1 )
  {
    shdiaph[i] = "UNDEF"
    list = avelist
    buf7 = ""
    while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != EOF )
    {
      if( buf == fn[i] )
      {
        shdiaph[i] = buf1
	cmnt[i] = buf7
	break
      }
      buf7 = ""
    }
  }

  printf( "Following tables are found\n" )
  for( i=1; i<=n_list; i+=1 )
    printf( "%2d :  %s %s %s\n", i, fn[i], shdiaph[i], cmnt[i] )

  printf( "Select for %s group\n", fn_lave )

nextstep1:

  printf( "Input table number ([0]: end lower-hole) >> " )
  while( scan( buf ) != 1 ){}
  if( buf == "0" )
    goto upperhole
  c = int( buf )
  if( c < 0 || c > n_list )
    goto nextstep1
  fn_sp = fn[c]

  pa_cel = -9999.0
  list = fn_spdat
  while( fscan( list, buf, buf1, buf2, buf3 ) != EOF )
  {
    if( substr( fn_sp, 2, 4 ) == substr( buf, 4, 6 ) )
    {
      pa_cel = real( buf3 )
      printf( "%s  %6.2f\n", fn_sp, pa_cel )
      break
    }
  }
  if( pa_cel < -1000.0 )
  {
    printf( "PA data of '%s' not found.\n", fn_sp )
    printf( "Input celetial position angle of %s >> ", fn_sp )
    while( scan( buf ) != 1 ){}
    pa_cel = real( buf )
  }

#  add  2004 11/20
#  printf("%s %s %s \n",fn_sp,gtlfile,uplfile)
#  printf("%s %s %s \n",sw,ew,pa_cel)
#  printf("%s \n",tmp_fn)
#  printf("\n")
#

#  printf("\n fn_sp=%s tmp_fn=%s \n",fn_sp,tmp_fn)
  trithorg( fn_sp, gtlfile, uplfile, sw, ew, pa_cel, tmp_fn )
#  printf("\n fn_sp=%s tmp_fn=%s \n",fn_sp,tmp_fn)
  list = tmp_fn
  if( fscan( list, pa_cel, pa_ins, pa_diff ) != 3 )
  {
    printf( "Error in 'trithorg'\n" )
    bye
  }
  printf( "%s  %s %6.2f %6.2f %6.2f\n", diaph, fn_sp, pa_cel, pa_ins, pa_diff )
  printf( "%s  %s %6.2f %6.2f %6.2f\n", diaph, fn_sp, pa_cel, pa_ins, pa_diff,\
          >> lfile )
  printf( "%s %6.2f %6.2f %6.2f\n", fn_sp, pa_cel, pa_ins, pa_diff,\
          >> ltmp )
  delete( tmp_fn, ver- )
  flag_lsel += 1
  
  goto nextstep1

upperhole:

#----- upper hole

  flag_usel = 0

  sections( uspfileid, option="fullname", > tmp_list )
  n_list = sections.nimage
  if( n_list == 0 )
  {
    printf( "Cannot find any table. Skip upper hole.\n" )
    delete( tmp_list, ver- )
    goto averaging
  }

  i = 0
  list = tmp_list
  while( fscan( list, buf ) != EOF )
  {
    i += 1
    fn[i] = buf
  }
  delete( tmp_list, ver- )

  for( i=1; i<=n_list; i+=1 )
  {
    shdiaph[i] = "UNDEF"
    buf7 = ""
    list = avelist
    while( fscan( list, buf, buf1, buf2, buf3, buf4, buf5, buf6, buf7 ) != EOF )
    {
      if( buf == fn[i] )
      {
        shdiaph[i] = buf1
	cmnt[i] = buf7
	break
      }
      buf7 = ""
    }
  }

  printf( "Following tables are found\n" )
  for( i=1; i<=n_list; i+=1 )
    printf( "%2d :  %s %s %s\n", i, fn[i], shdiaph[i], cmnt[i] )

  printf( "Select for %s group\n", fn_uave )

nextstep2:

  printf( "Input table number ([0]: end upper-hole) >> " )
  while( scan( buf ) != 1 ){}
  if( buf == "0" )
    goto averaging
  c = int( buf )
  if( c < 0 || c > n_list )
    goto nextstep2

  fn_sp = fn[c]

  pa_cel = -9999.0
  list = fn_spdat
  while( fscan( list, buf, buf1, buf2, buf3 ) != EOF )
  {
    if( substr( fn_sp, 2, 4 ) == substr( buf, 4, 6 ) )
    {
      pa_cel = real( buf3 )
      printf( "%s  %6.2f\n", fn_sp, pa_cel )
      break
    }
  }
  if( pa_cel < -1000.0 )
  {
    printf( "PA data of '%s' not found.\n", fn_sp )
    printf( "Input celetial position angle of %s >> ", fn_sp )
    while( scan( buf ) != 1 ){}
    pa_cel = real( buf )
  }

  trithorg( fn_sp, gtufile, upufile, sw, ew, pa_cel, tmp_fn )
  list = tmp_fn
  if( fscan( list, pa_cel, pa_ins, pa_diff ) != 3 )
  {
    printf( "Error in 'trithorg'\n" )
    bye
  }
  printf( "%s  %s %6.2f %6.2f %6.2f\n", diaph, fn_sp, pa_cel, pa_ins, pa_diff )
  printf( "%s  %s %6.2f %6.2f %6.2f\n", diaph, fn_sp, pa_cel, pa_ins, pa_diff,\
          >> lfile )
  printf( "%s %6.2f %6.2f %6.2f\n", fn_sp, pa_cel, pa_ins, pa_diff,\
          >> utmp )
  delete( tmp_fn, ver- )
  flag_usel += 1
  
  goto nextstep2


averaging:

#---- average the difference


  #--- lower

  if( flag_lsel > 0 )
  {
    pa_diff_ave = 0.0
    n = 0

   list = ltmp
   while( fscan( list, buf, pa_cel, pa_ins, pa_diff ) == 4 )
   {
     pa_diff_ave += pa_diff
     n += 1
   }
   delete( ltmp, ver- )
   if( n > 0 )
   {
    pa_diff_ave = pa_diff_ave / n
      printf( "%7.2f\n", pa_diff_ave, >> fn_lave )
      printf( "Lower average  %7.2f\n", pa_diff_ave  )
   }
  }

  #--- upper

  if( flag_usel > 0 )
  {
    pa_diff_ave = 0.0
    n = 0

    list = utmp
    while( fscan( list, buf, pa_cel, pa_ins, pa_diff ) == 4 )
    {
      pa_diff_ave += pa_diff
      n += 1
    }
    delete( utmp, ver- )
    if( n > 0 )
    {
      pa_diff_ave = pa_diff_ave / n
      printf( "%7.2f\n", pa_diff_ave, >> fn_uave )
      printf( "Upper average  %7.2f\n", pa_diff_ave  )
    }
  }

end
