#
#       TRI REDUCTION SUB ROUTINE
#
#       Averaging all spectra of UP standards : triupsum1.cl
#
#       1997/03/31      K.Kawabata Ver.0.10 for CRYOCAM manual reduc
#       1997/09/08      K.Kawabata Ver.1.00 e/o --> e, o, respectively
#
#

procedure triupsum1

begin

  file listoflist
  string trilist[250]
  string shorttyp[250], title[250]
  string exptime[250], diaph[250], filter[250]
  string slit[250]
  int upgrno # group number of UP standard

  file fn, fn_e, fn_o, fn_eoo
  string strdiaph

  file tmp_list, tmp_fn, tmp_fn1, tmp_fn2, tmp_fn3
  string tmp_e[16], tmp_o[16]
  int flag_theta[16]
  int c, i, j, n_list
  string buf = ""
  string buf1, buf2, buf3, buf4, buf5, buf6, buf7
  real theta
  real rsigma = 500.0 # no reject, practically
  int x0, x1, x2

  listoflist = envget( "_rootlist" )

  tmp_list = mktemp( "tri.tmp.list." )
  tmp_fn1 = mktemp( "tri.tmp.fn." )
  tmp_fn2 = mktemp( "tri.tmp.fn." )
  tmp_fn3 = mktemp( "tri.tmp.fn." )

  for( i=1; i<=16; i+=1 )
  {
    tmp_e[i] = mktemp( "tri.tmp.list." )
    tmp_o[i] = mktemp( "tri.tmp.list." )
  }

  print( "" )
  print( "------------------------------------------------" )
  print( " Averaging spectra of in each UP standard group" )
  print( "------------------------------------------------" )
  print( "" )

#---- erase old images

  sections( "upave*.fits", option="fullname", > tmp_list )
  if( sections.nimages > 0 )
  {
    imdelete( "@"//tmp_list, ver- )
    delete( tmp_list, ver- )
  }

#----- Scanning the list file

  if( access( listoflist ) == 0 )
  {
    print( "Cannot find "//listoflist//"   Please do trigroup" )
    return
  }
  n_list = 0
  list = listoflist
  while( fscan(list, buf1, buf2, buf3, buf4, buf5, buf6, buf7) != EOF )
  {
    if( substr( buf1, 1, 3 ) != "TRI" )
    {
      print( "The list-file ", listoflist, " is not produced by TRIGROUP.CL" )
      break
    }
    if( buf2 != "OBJ" || buf3 == "SKY" || buf3 == "sky" )
      next
    n_list += 1
    trilist[n_list] = buf1
    shorttyp[n_list] = buf2
    title[n_list] = buf3
    exptime[n_list] = buf4
    diaph[n_list] = buf5
    filter[n_list] = buf6
    slit[n_list] = buf7
  }

mainstart:
  
#---- selection of images for average

  printf( "\n" )
  for( i=1; i<=n_list; i+=1 )
  {
    printf( "%2d: %16s  %6s  %s  %s\n", i, trilist[i], title[i],\
            diaph[i], filter[i] )
  }

nextstep1:

  printf( "Choose a group number of UP standard ([0]:end) >> " )
  while( scan( buf ) != 1 ){}
  if( buf == "0" )
  {
#    printf("Now imstating...1 \n")
    imstat( "TRI*eoo.?.fits", format-, field="image,mean,stddev,midpt" )
    return
  }
  upgrno = int( buf )
  if( upgrno < 1 || upgrno > n_list )
    goto nextstep1

#----- diaphragm check

  strdiaph = diaph[upgrno]

#----- sort by its POS-HWP value

  sed( 's/fits//', trilist[upgrno], >> tmp_list )

#  sed( 's/fit\.fits//', trilist[upgrno] )

  list = tmp_list

  while( fscan( list, fn ) != EOF )
  {
    if( access( fn//"od.e.fits" ) == 0 || access( fn//"od.o.fits" ) == 0 )
    {
      printf( "Cannot find %s.\n", fn )
      printf("hogehoge\n")
      next
    }

    imgets( fn//"od.e", "POS-HWP" )
    theta = real( imgets.value )
    i = int( theta / 22.4999 ) + 1
    imgets( fn//"od.e", 'i_naxis1' )
    x0 = int( imgets.value )
    x1 = 0.35 * x0
    x2 = 0.65 * x0

    flag_theta[i] = flag_theta[i] + 1
    tmp_fn = tmp_e[i]
    print( fn//"od.e", >> tmp_fn )
    tmp_fn = tmp_o[i]
    print( fn//"od.o", >> tmp_fn )
  }

  delete( tmp_list, ver- )

#---- averaging

  for( i=1; i<=16; i+=1 ){

    fn_e = trilist[upgrno]//"."//strdiaph//"e."//(i-1)
    fn_o = trilist[upgrno]//"."//strdiaph//"o."//(i-1)
    fn_eoo = trilist[upgrno]//"."//strdiaph//"eoo."//(i-1)

    if( access( fn_e//".fits" ) )
      imdelete( fn_e, ver- )
    if( access( fn_o//".fits" ) )
      imdelete( fn_o, ver- )
    if( access( fn_eoo//".fits" ) )
      imdelete( fn_eoo, ver- )

    if( flag_theta[i] > 0 )
    {

      triaverage( tmp_e[i], "none", x1, x2, fn_e, tmp_fn1, tmp_fn2, tmp_fn3, rsigma )

      imdelete( tmp_fn1, ver- )
      if( access( tmp_fn2 ) == 1 )
        delete( tmp_fn2, ver- )
      if( access( tmp_fn3 ) == 1 )
        delete( tmp_fn3, ver- )

      triaverage( tmp_o[i], "none", x1, x2, fn_o, tmp_fn1, tmp_fn2, tmp_fn3, rsigma )

      imdelete( tmp_fn1, ver- )
      if( access( tmp_fn2 ) == 1 )
        delete( tmp_fn2, ver- )
      if( access( tmp_fn3 ) == 1 )
        delete( tmp_fn3, ver- )

      imarith( fn_e, "/", fn_o, fn_eoo, pixtype="real", \
               calctype="real", ver- )

      imreplace( fn_eoo, 0.0, lower=INDEF, upper=0.0 )
      imreplace( fn_eoo, 3.0, lower=3.0, upper=INDEF )

    }
  }

  for( i=1; i<=16; i+=1 ){
    if( flag_theta[i] > 0 )
    {
      delete( tmp_e[i], ver- )
      delete( tmp_o[i], ver- )
    }
  }

  goto mainstart

end
